;;;; calling.el -- part of John's elisp change-ringing system
;;; Time-stamp: <98/06/18 09:40:30 john>

;;; Calling practice for the user

(provide 'calling)
(require 'method-groups)
(require 'touch-book)			; for ringing:choose-action

(defun ringing:user-lead-end-call ()
  "Ask the user for a type of lead end."
  (ringing:choose-action "Lead end: " '(plain bob single)))

(defun ringing:user-choose-method (prompt methods &optional default)
  "With PROMPT, let the user choose between METHODS.
Optional DEFAULT."
  (completing-read prompt methods nil nil default))

(defun ringing:user-calling-helper-function (row
					     remaining-changes
					     method-holder
					     rounds
					     place-marker
					     methods)
  "Helper function for ringing with the user calling."
  (if (string= row rounds)
      nil
    (if remaining-changes
	t
      (let ((method (car method-holder)))
	(cons row
	      (case (car place-marker)
		((lead-end)
		 (setq method (ringing:find-named-method
			       (ringing:user-choose-method "Method: "
							   methods
							   (ringing:method-name method))))
		 (rplaca method-holder method)
		 (rplaca place-marker 'work)
		 (change-ringing-method-work method))
		((work)
		 (rplaca place-marker 'lead-end)
		 (case (ringing:user-lead-end-call)
		   (plain (change-ringing-method-plain-lead-end method))
		   (bob (change-ringing-method-bob method))
		   (single (change-ringing-method-single method))))))))))

(defvar ringing:user-calling-show-whole-leads nil
  "*Whether to show the whole lead, or just the lead end, in calling practice.")

(defun ringing:user-calling-display-function (row
					      remaining-changes
					      method-holder
					      rounds
					      place-marker)
  "Display the lead end (and possibly the whole lead leading up to it)."
  (when (or ringing:user-calling-show-whole-leads
	    (and (null (cdr remaining-changes))))
    (if (eq (car place-marker) 'work)
	(insert ?\n))
    (ringing:apply-bell-colours row)
    (ringing:insert-row-lines row)))

(defun ringing:call-touch (group)
  "Give the ringer practice calling touches from GROUP."
  (interactive
   (list (completing-read "Call a touch from group: "
			  (method-groups:all-groups)
			  nil
			  t
			  nil)))
  (let* ((group-methods (method-groups:group-methods group))
	 (method (ringing:find-named-method (ringing:user-choose-method "Go: " group-methods)))
	 (method-holder (list method))
	 (place-marker (list 'work))
	 (ringing:end-of-row-char ?\n))
    (switch-to-buffer (ringing:get-buffer-create "*Calling practice*"))
    (erase-buffer)
    (ringing:ring-from-rounds method
			      'ringing:user-calling-helper-function
			      (list place-marker group-methods)
			      'ringing:user-calling-display-function
			      (list place-marker))))

;;; end of calling.el

