;;;; classification.el -- part of John's elisp change-ringing system
;;; Time-stamp: <99/01/19 13:45:55 johnstu>

;;; investigate some characteristics of a method, such as:
;;;   is it plain?
;;;         treble bob, surprise, delight?
;;;         a principle? multiple hunt?
;;;         symmetrical?
;;;         regular?
;;;         double?
;;;   what is its lead end order?

(provide 'classification)
(require 'place-notation)

(defvar ringing:plain-bob-lead-ends (make-vector 16 nil)
  "An array containing arrays of the lead ends for Plain Bob at each stage.")

(defun ringing:plain-bob-lead-ends (stage)
  "Return an array containing the lead ends for Plain Bob at STAGE."
  (when (stringp stage) (setq stage (ringing:figure-out-number-of-bells stage)))
  (when (null (aref ringing:plain-bob-lead-ends stage))
    (aset ringing:plain-bob-lead-ends stage
	  (ringing:plain-course-leads
	   (ringing:find-named-method
	    (concat "Plain Bob "
		    (ringing:stage-name stage))))))
  (aref ringing:plain-bob-lead-ends stage))

(defun ringing:method-regular-p (method)
  "Determine whether METHOD is regular.
The value returned is the number of leads of plain bob to get to the first
lead head of METHOD, if METHOD is regular, or nil if method is not regular."
  (interactive
   (let* ((method-name (ringing:read-method-name "Determine regularity of method: ")))
     (list method-name)))
  (let* ((method-data (ringing:find-named-method method))
	 (lead-ends (ringing:plain-course-leads method-data))
	 (is-regular (position (car lead-ends)
			     (ringing:plain-bob-lead-ends (change-ringing-method-nbells method-data))
			     :test 'string=
			     )))
    (when (interactive-p)
      (message "%s %s regular" (ringing:method-name method) (if is-regular "is" "is not")))
    is-regular))

;;; end of classification.el
