;;;; composition-search.el -- part of John's elisp change-ringing system
;;; search tree of possible compositions
;;; Time-stamp: <98/01/31 14:44:59 john>

(eval-when (compile load eval)
  (provide 'composition-search)
  (require 'composition-checker)
  (require 'method-groups))

;;;; search composition trees

(defun ringing:try-next-leads (touch-so-far method-group-calls)
  "Make list of true touches from adding to TOUCH-SO-FAR each of METHOD-GROUP-CALLS.
Each element of the result is a cons of the method call tried and the resulting
touch-so-far vector."
  (let ((result nil))
    (message "Starting ringing:try-next-leads from %s" touch-so-far)
    (dolist (method-call method-group-calls)
      (message " Trying %s" method-call)
      (let ((try (ringing:check-touch method-call
				      nil
				      touch-so-far)))
	(message "  resulting in %s" try)
	(when (first try)
	  (push (cons method-call (second try))
		result))))
        (message "Finishing ringing:try-next-leads")
    result))

(setq finalposses nil)

(defun ringing:filter-final-completions (possibles rounds)
  "Return a list containing members of POSSIBLES that end in ROUNDS.
ROUNDS might not really be rounds, although it normally would be -- it is
actually whatever change you want to stop at."
  (push possibles finalposses)
  (let ((result nil))
    (dolist (possible possibles)
      ;; possible is a cons of
      ;;   the method call tried
      ;;   the touch-so-far vector
      (when (ringing:roundsp
	     (car (last (aref (cdr possible) 0)))
	     rounds)
	(push possible result)))
    result))

(defconst ringing:just-a-bob (touch bob)
  "A touch consisting of just a bob.")

(defun ringing:make-method-calls (method-group)
  "Derive a list of touch specifications from a method group such as that
returned by method-groups:find-named-group."
  (mapcar (function
	   (lambda (method-in-group)
	     (ringing:normalize-touch (list 'method (cdr method-in-group)))))
	  (method-groups:method-group-methods method-group)))

(defun ringing:true-completions-for-touch (touch-so-far
					   method-group-calls
					   &optional max-further-leads
					   helper-function
					   offered-rounds)
  "Return a list of true completions for TOUCH-SO-FAR using METHOD-GROUP-CALLS.
Optional MAX-FURTHER-LEADS limits the number of leads to add.
An optional HELPER-FUNCTION may be given (to handle anything funny about the
methods.
TOUCH-SO-FAR should be a vector, as described for ringing:check-touch,
which see. METHOD-GROUP-CALLS should be a list of touch specifications as
derived by ringing:make-method-calls. If a method-group is given instead, it
is converted.
Another argument OFFERED-ROUNDS may be given to save regenerating a rounds
string on each call. This could also be used to stop the completion generator
at something other than rounds."
  (when (method-groups:method-group-p method-group-calls)
    (setq method-group-calls (ringing:make-method-calls method-group-calls)))
  (message "method-group-calls is %s" method-group-calls)
  (message "(car method-group-calls) is %s" (car method-group-calls))
  (message "Touch-So-Far is %s" touch-so-far)
  (message "(aref touch-so-far 0) is %s" (aref touch-so-far 0))
  (message "(car (aref touch-so-far 0)) is %s" (car (aref touch-so-far 0)))
  (let* ((ringing:end-of-row-char ? )
	 (nbells (if (stringp (car (aref touch-so-far 0)))
		     (ringing:n-bells-from-row-string (car (aref touch-so-far 0)))
		   (if offered-rounds
		       (ringing:n-bells-from-row-string offered-rounds)
		     (change-ringing-method-nbells (third (car method-group-calls))))))
	 (rounds (if offered-rounds offered-rounds (ringing:make-rounds nbells)))
	 ;; each of these is [a list of] (touchcontrol . (truth [ rowlist currentmethod ] ) )
	 (with-bob (cons ringing:just-a-bob (ringing:check-touch ringing:just-a-bob nil touch-so-far)))
	 (with-each-method (ringing:try-next-leads touch-so-far method-group-calls))
	 (with-bob-and-each-method
	  (if (cadr with-bob)
	      (ringing:try-next-leads (caddr with-bob) method-group-calls)
	    nil))
	 (all-true-single-completions (cons with-bob
					    (append
					     with-each-method
					     with-bob-and-each-method)))
	 (all-true-final-single-completions (ringing:filter-final-completions
					     all-true-single-completions
					     rounds))
	 (further nil)
	 )
    (message "Search with up to %d leads to go" max-further-leads)
    (message "with-bob = %s" with-bob)
    (message "with-each-method = " with-each-method)
    (message "with-bob-and-each-method = %s" with-bob-and-each-method)
    (message "all-true-single-completions = %s" all-true-single-completions)
    (message "all-true-final-single-completions = %s" all-true-final-single-completions)
    (when (or (null max-further-leads) (and (numberp max-further-leads) (> max-further-leads 1)))
      (setq further
	    nil				; whatever
	    ))
    (append all-true-final-single-completions further)))

;; end of composition-search.el
