;; custom-print.el -- Provide a dialogue box like interface for printing.

(provide 'custom-print)
(require 'custom)
(require 'mSiRiL-mode)
(require 'ps-line)
(require 'method-groups)
(require 'stages)

;; John you should be using a global variable like this I think. You
;; can put it in the right place. I don't think it belongs here.
(defvar ringing:keep-postscript-file t
  "*If non-nil, the postscript file will not be deleted after
sending the job to the printer.")

(defvar custom-print:selected-method (make-marker)
  "Marker for the line in the msiril file from which
`mSiRiL-mode:custom-print' was invoked.")

(defvar custom-print:group nil
  "If non-nil, the group to be printed, otherwise no group.")

(defvar custom-print:printed-title ""
  "The title to print with a group style.")

(defvar custom-print:custom-local-type-properties nil
  "`custom-local-type-properties' used in custom-print.")

;; One entry point for both single method printing and groups.
(defun mSiRiL-mode:custom-print ()
  "Provide a dialogue box like interface for printing."
  (interactive)
  (set-marker custom-print:selected-method (point))
  (switch-to-buffer-other-window (current-buffer))
  ;; reset data before next try...
  (let ((printing-custom-data
         '((tag . "Method Printing in PostScript")
           (type . group)
           (data ((tag . "\n")
                  (header . nil)
                  (compact . t)
                  (type . group)
                  (doc . "
Press \[Print\]   to accept the current settings, print the method and exit,
Press \[Preview\] to accept the current settings and preview the method in PostScript,
Press \[Apply\]   to set the default values of the fields without printing or previewing,
Press \[Reset\]   to discard any changes made since last `print', `preview' or `apply'.
")
                  (data ((tag . "Print")
                         (type . button)
                         (query . mSiRiL-mode:custom-print-button))
                        ((tag . "Preview")
                         (type . button)
                         (query . mSiRiL-mode:custom-preview-button))
                        ((tag . "Apply")
                         (type . button)
                         (query . custom-print:custom-apply-all))
                        ((tag . "Reset")
                         (type . button)
                         (query . custom-reset-all))
                        ))
                 ((type . group)
                  (tag . "Printing Configuration.")
                  (data ((tag . "Title Style")
                         (name . ringing:latest-ps-title-style)
                         (default . "default")
                         (type . title-style-choose))
                        ((tag . "Blue Line Style")
                         (name . ringing:latest-ps-style)
                         (default . "default")
                         (type . line-style-choose))
                        ((tag . "Grid Style")
                         (name . ringing:latest-ps-grid-style)
                         (default . "none")
                         (type . grid-style-choose))
                        ((tag . "Group")
                         (name . custom-print:group)
                         (default . nil)
                         (type . group-choose))
                        ((tag . "Group Style")
                         (name . ringing:latest-ps-group-style)
                         (default . "default")
                         (type . group-style-choose))
                        ((tag . "Group Title")
                         (name . custom-print:printed-title)
                         (default . "")
                         (type . string))
                        ((tag . "Keep PostScript File")
                         (name . ringing:keep-postscript-file)
                         (default . t)
                         (type . toggle))
                        ))
                 ((tag . "\n")
                  (header . nil)
                  (compact . t)
                  (type . group)
                  (data ((tag . "Print")
                         (type . button)
                         (query . mSiRiL-mode:custom-print-button))
                        ((tag . "Preview")
                         (type . button)
                         (query . mSiRiL-mode:custom-preview-button))
                        ((tag . "Apply")
                         (type . button)
                         (query . custom-print:custom-apply-all))
                        ((tag . "Reset")
                         (type . button)
                         (query . custom-reset-all))
                        ))
                 ))))
    ;; Must be updated each time!
    (setq custom-print:custom-local-type-properties
          `((group-choose (type . choice)
                          (data ((tag . "None")
                                 (default . nil)
                                 (type . const))
                                ,@(mapcar '(lambda (x)
                                             `((tag . ,(car x))
                                               (default . ,(car x))
                                               (type . const)))
                                          (method-groups:all-groups))))
            (group-style-choose (type . choice)
                                (data ,@(mapcar '(lambda (x)
                                                   `((tag . ,(car x))
                                                     (default . ,(car x))
                                                     (type . const)))
                                                (ringing:ps-group-styles))))
            (title-style-choose (type . choice)
                                (data ,@(mapcar '(lambda (x)
                                                   `((tag . ,(car x))
                                                     (default . ,(car x))
                                                     (type . const)))
                                                (ringing:ps-title-styles))))
            (line-style-choose (type . choice)
                               (data ,@(mapcar '(lambda (x)
                                                  `((tag . ,(car x))
                                                    (default . ,(car x))
                                                    (type . const)))
                                               (ringing:ps-styles))))
            (grid-style-choose (type . choice)
                               (data ,@(mapcar '(lambda (x)
                                                  `((tag . ,(car x))
                                                    (default . ,(car x))
                                                    (type . const)))
                                               (ringing:ps-grid-styles))))
            (number (type . sexp)
                    (width . 20)
                    (valid . (lambda (c d) (numberp d))))))
    (let ((custom-local-type-properties custom-print:custom-local-type-properties))
      (custom-initialize printing-custom-data))
    (custom-buffer-create "*Customize*"
                          printing-custom-data
                          custom-print:custom-local-type-properties)
    (custom-reset-all))
  (search-forward "Method Printing in PostScript")
  (beginning-of-line)
  (set-window-start (selected-window) (point)))

(defun mSiRiL-mode:custom-print-button ()
  "Call back function to actually print something."
  (interactive)
  (let ((custom-local-type-properties custom-print:custom-local-type-properties))
    (custom-apply-all)
    (let ((buf (get-buffer "*Customize*")))
      (delete-window (get-buffer-window buf nil))
      (kill-buffer buf))
    (if custom-print:group
        (method-group:print-group custom-print:group
                                  ringing:latest-ps-style
                                  (ringing:prompt-for-style-parameters
                                   (expand-file-name ringing:latest-ps-style
                                                     ringing:postscript-styles-dir))
                                  ringing:latest-ps-title-style
                                  (ringing:prompt-for-style-parameters
                                   (expand-file-name ringing:latest-ps-title-style
                                                     ringing:postscript-group-styles-dir))
                                  ringing:latest-ps-grid-style
                                  (ringing:prompt-for-style-parameters
                                   (expand-file-name ringing:latest-ps-grid-style
                                                     ringing:postscript-grid-styles-dir))
                                  ringing:latest-ps-group-style
                                  (ringing:prompt-for-style-parameters
                                   (expand-file-name ringing:latest-ps-group-style
                                                     ringing:postscript-group-styles-dir))
                                  custom-print:printed-title)
      (mSiRiL-mode:print custom-print:selected-method
                         (prefix-numeric-value current-prefix-arg)
                         ringing:latest-ps-style
                         (ringing:prompt-for-style-parameters
                          (expand-file-name ringing:latest-ps-style
                                            ringing:postscript-styles-dir))
                         ringing:latest-ps-title-style
                         (ringing:prompt-for-style-parameters
                          (expand-file-name ringing:latest-ps-title-style
                                            ringing:postscript-title-styles-dir))
                         ringing:latest-ps-grid-style
                         (ringing:prompt-for-style-parameters
                          (expand-file-name ringing:latest-ps-grid-style
                                            ringing:postscript-grid-styles-dir))))
    (set-marker custom-print:selected-method nil)))

(defun mSiRiL-mode:custom-preview-button ()
  "Call back function to preview something."
  (interactive)
  (let ((custom-local-type-properties custom-print:custom-local-type-properties))
    (custom-apply-all)
    (save-excursion
      (set-buffer (marker-buffer custom-print:selected-method))
      (if custom-print:group
          (method-group:preview-group custom-print:group
                                      ringing:latest-ps-style
                                      (ringing:prompt-for-style-parameters
                                       (expand-file-name ringing:latest-ps-style
                                                         ringing:postscript-styles-dir))
                                      ringing:latest-ps-title-style
                                      (ringing:prompt-for-style-parameters
                                       (expand-file-name ringing:latest-ps-title-style
                                                         ringing:postscript-group-styles-dir))
                                      ringing:latest-ps-grid-style
                                      (ringing:prompt-for-style-parameters
                                       (expand-file-name ringing:latest-ps-grid-style
                                                         ringing:postscript-grid-styles-dir))
                                      ringing:latest-ps-group-style
                                      (ringing:prompt-for-style-parameters
                                       (expand-file-name ringing:latest-ps-group-style
                                                         ringing:postscript-group-styles-dir))
                                      custom-print:printed-title)
        (mSiRiL-mode:preview custom-print:selected-method
                             (prefix-numeric-value current-prefix-arg)
                             ringing:latest-ps-style
                             (ringing:prompt-for-style-parameters
                              (expand-file-name ringing:latest-ps-style
                                                ringing:postscript-styles-dir))
                             ringing:latest-ps-title-style
                             (ringing:prompt-for-style-parameters
                              (expand-file-name ringing:latest-ps-title-style
                                                ringing:postscript-title-styles-dir))
                             ringing:latest-ps-grid-style
                             (ringing:prompt-for-style-parameters
                              (expand-file-name ringing:latest-ps-grid-style
                                                ringing:postscript-grid-styles-dir)))))))

(defun custom-print:custom-apply-all ()
  (interactive)
  (let ((custom-local-type-properties custom-print:custom-local-type-properties)
        (buf (get-buffer "*Customize*")))
    (custom-apply-all)
    (delete-window (get-buffer-window buf nil))
    (kill-buffer buf))
  (set-marker custom-print:selected-method nil))
