;;;; mSiRiL-mode.el -- part of John's elisp change-ringing system
;;; major mode for microSiRiL files
;;; Time-stamp: <99/11/03 09:55:25 johnstu>

(eval-when (compile load eval)
  (require 'bell-colours)
  (provide 'mSiRiL-mode)
  (require 'stages)
  (require 'method-library)
  (require 'ringing-methods)
  (require 'method-groups)
  (require 'plain-courses)
  (require 'bell-colours)
  (require 'method-display)
  (require 'leads)
  (require 'above-below)
  (require 'touch-book))

;;;; This file defines a major mode for microSiRiL method library files.

;;; Main entry points:
;;; mSiRiL-mode
;;; mSiRiL-mode:lines-for-method
;;; mSiRiL-mode:popup-combined-method-lines
;;; mSiRiL-mode:ring-method
;;; mSiRiL-mode:buffer-ephemeral-group
;;; mSiRiL-mode:buffer-group
;;; mSiRiL-mode:insert-description

(makunbound 'mSiRiL-keymap)
(defvar mSiRiL-keymap
  (make-keymap "mSiRiL")
  "Keymap for mSiRiL mode.")

(suppress-keymap mSiRiL-keymap)

(define-key mSiRiL-keymap "\r" 'mSiRiL-mode:ring-method)
(define-key mSiRiL-keymap "H" 'mSiRiL-mode:handbells-ring-method)
(define-key mSiRiL-keymap "?" 'mSiRiL-mode:popup-method-lines)
(define-key mSiRiL-keymap "h" 'mSiRiL-mode:popup-method-both-handbell-lines)
(define-key mSiRiL-keymap "&" 'mSiRiL-mode:popup-combined-method-lines)
(define-key mSiRiL-keymap "r" 'mSiRiL-mode:teach-random-lead-of-method)
(define-key mSiRiL-keymap "R" 'mSiRiL-mode:teach-random-leads-of-method)
(define-key mSiRiL-keymap "\M-e" 'mSiRiL-mode:full-grid-edit)
(define-key mSiRiL-keymap "x" 'mSiRiL-mode:full-grid-explore)
(define-key mSiRiL-keymap "p" 'mSiRiL-mode:print)
(define-key mSiRiL-keymap "P" 'method-group:print-group)
(define-key mSiRiL-keymap "v" 'mSiRiL-mode:preview)
(define-key mSiRiL-keymap "V" 'method-group:preview-group)
(define-key mSiRiL-keymap "i" 'mSiRiL-mode:intensive-teaching)
(define-key mSiRiL-keymap "l" 'touch-book:list-my-methods)
(define-key mSiRiL-keymap "e" 'mSiRiL-mode:eleven)
(define-key mSiRiL-keymap "E" 'mSiRiL-mode:eleven)
(define-key mSiRiL-keymap "t" 'mSiRiL-mode:twelve)
(define-key mSiRiL-keymap "T" 'mSiRiL-mode:twelve)
(define-key mSiRiL-keymap "a" 'mSiRiL-mode:thirteen)
(define-key mSiRiL-keymap "A" 'mSiRiL-mode:thirteen)
(define-key mSiRiL-keymap "b" 'mSiRiL-mode:fourteen)
(define-key mSiRiL-keymap "B" 'mSiRiL-mode:fourteen)
(define-key mSiRiL-keymap "c" 'mSiRiL-mode:fifteen)
(define-key mSiRiL-keymap "C" 'mSiRiL-mode:fifteen)
(define-key mSiRiL-keymap "f" 'mSiRiL-mode:sixteen)
(define-key mSiRiL-keymap "F" 'mSiRiL-mode:sixteen)
(define-key mSiRiL-keymap "w" 'touch-book:worst-leads-first)
(define-key mSiRiL-keymap "W" 'touch-book:teach-worst-lead)
(define-key mSiRiL-keymap "*" 'touch-book:revise-my-methods)
(define-key mSiRiL-keymap "m" 'mSiRiL-mode:toggle-method-mark)
(define-key mSiRiL-keymap "M" 'mSiRiL-mode:clear-method-marks)
(define-key mSiRiL-keymap "=" 'mSiRiL-mode:methods-with-same-changes)
(define-key mSiRiL-keymap "G" 'method-group:make-group-buffer)

;; the buffer-local variables

(defvar mSiRiL-nbells nil
  "Per-buffer variable for mSiRiL-mode, giving number of bells, as number.")

(defvar mSiRiL-stage-name nil
  "Per-buffer variable for mSiRiL-mode, giving number of bells, as string.")

(defvar mSiRiL-method-kind nil
  "Per-buffer variable for mSiRiL-mode, giving kind of method, as symbol.")

(defvar mSiRiL-method-kind-name nil
  "Per-buffer variable for mSiRiL-mode, giving kind of method, as string.")

(defvar mSiRiL-ephemeral-group nil
  "List of methods marked in this buffer, each dotted with its definition.")

;; suggested auto-mode-alist entry:
;; ("method-libraries" . mSiRiL-mode)

(defvar mSiRiL-mode-load-hook nil
  "Hook to run on loading mSiRiL mode.")

(defvar mSiRiL-mode-hook nil
  "Hook to run when putting a buffer into mSiRiL mode.")

(defvar mSiRiL:do-own-coloring t
  "*Whether mSiRiL-mode should do its own coloring.")

(defvar mSiRiL:color-lazily t
  "*Whether mSiRiL-mode should do its coloring lazily.")

(defvar mSiRiL-nmethods nil
  "The number of methods in this library file, as a number.")

(defvar mSiRiL-nmethods-string nil
  "The number of methods in this library file, as a string, used in the mode line.")

(put 'mSiRiL-mode 'mode-class 'special)

(defun mSiRiL-mode ()
  "Major mode for microSiRiL files.

Most commands work on the method at point:
  \\[mSiRiL-mode:popup-method-lines]	display it
  \\[mSiRiL-mode:ring-method]	try ringing it interactively
  \\[mSiRiL-mode:teach-random-leads-of-method]	test yourself on random leads of it
  \\[mSiRiL-mode:print]	print it via PostScript
  \\[mSiRiL-mode:preview]	preview PostScript
  \\[mSiRiL-mode:intensive-teaching]	learn it intensively
  \\[mSiRiL-mode:full-grid-explore]	explore its grid
  \\[mSiRiL-mode:full-grid-edit]	edit its grid

You can combine two methods (above and below the treble, at point and
mark respectively) with `\\[mSiRiL-mode:popup-combined-method-lines]'.

The ringing system keeps a logbook of what you have rung interactively (whether
from mSiRiL-mode or directly) and you can display that with `\\[touch-book:list-my-methods]'.

You can also display the leads you have tried, worst lead first, with `\\[touch-book:worst-leads-first]',
and do intensive learning on your current worst lead with `\\[touch-book:teach-worst-lead]'.

To revise the methods you've tried so far (selectively, starting with the
worst), use `\\[touch-book:revise-my-methods]'.

An ephemeral group of methods is kept for each buffer, which for most purposes (such
as printing a group, with `\\[method-group:print-group]') is like any other method group,
such as those defined in the directory method-groups:groups-directory.
Use `\\[mSiRiL-mode:toggle-method-mark]' to toggle whether the method at point is in the buffer's ephemeral group,
and \\[mSiRiL-mode:clear-method-marks] to remove all methods from the buffer's ephemeral group.

Commands available are:

\\{mSiRiL-keymap}

Typing digits specifies the starting place bell for commands that work
on whole courses, which is then remembered for that method; means ten,
E eleven, and T twelve.
"
  (interactive)
  (kill-all-local-variables)
  ;; (fundamental-mode)
  (mapcar 'make-local-variable
	  '(mSiRiL-nbells
	    mSiRiL-stage-name
	    mSiRiL-method-kind
	    mSiRiL-method-kind-name
	    mSiRiL-ephemeral-group
	    mSiRiL-nmethods mSiRiL-nmethods-string
	    window-scroll-functions))
  (setq major-mode 'mSiRiL-mode
	mode-name "mSiRiL"
	mSiRiL-nbells (ringing:figure-out-number-of-bells (buffer-file-name))
	mSiRiL-stage-name (ringing:stage-name mSiRiL-nbells)
	mSiRiL-method-kind (ringing:figure-out-kind (buffer-file-name))
	mSiRiL-method-kind-name (capitalize (ringing:kind-by-name mSiRiL-method-kind))
	mSiRiL-nmethods (1- (count-lines (point-min) (point-max)))
	mSiRiL-nmethods-string (format ": %d methods" mSiRiL-nmethods)
	mSiRiL-ephemeral-group nil
	;; although this is buffer-local, they start by sharing values!
	mode-line-format (copy-sequence mode-line-format)
	buffer-read-only t
	)
  (let ((mode-tail (memq 'mode-name mode-line-format)))
    (when mode-tail
      (rplacd mode-tail
	      (append '(" " mSiRiL-method-kind-name " " mSiRiL-stage-name mSiRiL-nmethods-string)
		      (cdr mode-tail)))))
  (use-local-map mSiRiL-keymap)
  (pushnew (current-buffer) ringing:buffers)
  (run-hooks 'mSiRiL-mode-hook)
  (when mSiRiL:do-own-coloring
    (if mSiRiL:color-lazily
	(mSiRiL-mode:start-lazy-coloring)
      (mSiRiL-mode:color-buffer))))

(defalias 'msiril-mode (symbol-function 'mSiRiL-mode))

(defun mSiRiL-mode:eleven ()
  "Set the prefix argument to 11."
  (interactive)
  (setq prefix-arg 11))

(defun mSiRiL-mode:twelve ()
  "Set the prefix argument to 12."
  (interactive)
  (setq prefix-arg 12))

(defun mSiRiL-mode:thirteen ()
  "Set the prefix argument to 13."
  (interactive)
  (setq prefix-arg 13))

(defun mSiRiL-mode:fourteen ()
  "Set the prefix argument to 14."
  (interactive)
  (setq prefix-arg 14))

(defun mSiRiL-mode:fifteen ()
  "Set the prefix argument to 15."
  (interactive)
  (setq prefix-arg 15))

(defun mSiRiL-mode:sixteen ()
  "Set the prefix argument to 16."
  (interactive)
  (setq prefix-arg 16))

(defun mSiRiL-mode:method-name-at (where)
  "Return the method name around WHERE."
  (save-excursion
    (goto-char where)
    (let* ((start (progn (beginning-of-line 1) (point)))
	   (end (progn (end-of-line 1) (point)))
	   (whole-method-string (buffer-substring start end)))
      (remove-text-properties 0 (length whole-method-string) '(face nil) whole-method-string)
      (if (string-match microSiRiL:line-parts-regexp whole-method-string)
	  (let* ((base-name (substring whole-method-string (match-beginning 1) (match-end 1))))
	    ;; do this one after all the match-data references, as it disturbs match-data
	    (microSiRiL:constructed-name base-name mSiRiL-method-kind-name mSiRiL-stage-name))
	(error "Cannot parse \"%s\" as a microSiRiL method definition" whole-method-string)))))

(defun mSiRiL-mode:method-at (where)
  "Return the method defined around WHERE."
  (save-excursion
    (goto-char where)
    (let* ((start (progn (beginning-of-line 1) (point)))
	   (end (progn (end-of-line 1) (point)))
	   (whole-method-string (buffer-substring start end)))
      (if (string-match microSiRiL:line-parts-regexp whole-method-string)
	  (let* ((base-name (substring whole-method-string (match-beginning 1) (match-end 1)))
		 (group-symbol (intern (substring whole-method-string (match-beginning 2) (match-end 2))))
		 (place-notation (substring whole-method-string (match-beginning 3) (match-end 3)))
		 ;; do this one after all the match-data references, as it disturbs match-data
		 (full-name (microSiRiL:constructed-name base-name mSiRiL-method-kind-name mSiRiL-stage-name))
		 (method-data-list (list full-name group-symbol place-notation))
		 (method-data (place-notation:as-method method-data-list)))
	    (ringing:store-method (car method-data-list)
				  (cdr method-data-list)
				  (buffer-file-name))
	    method-data)
	(error "Cannot parse \"%s\" as a microSiRiL method definition" whole-method-string)))))

(defvar mSiRiL-mode:ringing-in-progress-format "Ringing %s"
  "Format for how to say that ringing method (%s) is in progress")

(defvar mSiRiL-mode:method-header-format
  "%s:\n%s\nFully expanded: %s"
  "Header format for mSiRiL method display buffers.
For format called with
  method name
  place notation
  fully expanded place notation
  original description
as arguments (all as strings).")

(defvar mSiRiL-mode:marked-bells-for-methods nil
  "Which bells the user has specifically asked for as marked bells
for particular methods.")

(defun mSiRiL-mode:note-marked-bell-for-method (method-name bell)
  "Mark that for METHOD-NAME, BELL is the marked bell to use."
  (if (< bell 1)
      (progn
	(setq mSiRiL-mode:marked-bells-for-methods
	      (delete-if (function (lambda (pair) (equal (car pair) method-name)))
			 mSiRiL-mode:marked-bells-for-methods))
	nil)
    (let ((pair (assoc method-name mSiRiL-mode:marked-bells-for-methods)))
      (if pair
	  (rplacd pair bell)
	(push (cons method-name bell) mSiRiL-mode:marked-bells-for-methods)))
    bell))

(defun mSiRiL-mode:marked-bell-for-method (method-name)
  "Return the marked bell for METHOD-NAME, or nil if none marked."
  (cdr (assoc method-name mSiRiL-mode:marked-bells-for-methods)))

(defun mSiRiL-mode:choose-bell-to-mark (method-name &optional marked-bell-number)
  "Choose which bell to mark for METHOD-NAME.
If MARKED-BELL-NUMBER is given and is a number greater than 1, use that,
and remember it for future uses on this method.
Otherwise, take the value of MARKED-BELL-NUMBER previously given with this
METHOD-NAME, if any; otherwise the lowest bell that is making places at the
lead end, other than the treble. If no bell other than the treble is making
places at the lead end, return the tenor.
If a number less than 1 is given (remembering that 0 is taken as ten, so use
\\[universal-argument] - 1 for example) switch back to choosing automatically."
  (if (and (numberp marked-bell-number)
	   (> marked-bell-number 1))
      (mSiRiL-mode:note-marked-bell-for-method method-name marked-bell-number)
    (progn
      (when (and (numberp marked-bell-number) (< marked-bell-number 1))
	(mSiRiL-mode:note-marked-bell-for-method method-name marked-bell-number))
      (let ((method (ringing:find-named-method method-name t))
	    (old-marked (mSiRiL-mode:marked-bell-for-method method-name)))
	(if (numberp old-marked)
	    old-marked
	  (if (null ringing:observation-number)
	      (if method
		  (let ((lowest-places (ringing:lowest-working-bell-making-places
					(car (change-ringing-method-plain-lead-end method)))))
		    (if (numberp lowest-places)
			lowest-places
		      (change-ringing-method-nbells method)))
		mSiRiL-nbells)
	    ringing:observation-number))))))

(defvar displayed-method:method-name nil
  "The name of the method displayed in this buffer.")

(put 'displayed-method-mode 'mode-class 'special)

(makunbound 'displayed-method-keymap)
(defvar displayed-method-keymap
  (make-keymap "displayed-method")
  "Keymap for displayed-method mode.")

(suppress-keymap displayed-method-keymap)

(define-key displayed-method-keymap "?" 'displayed-method:display-similar-method)
(define-key displayed-method-keymap "\M-." 'displayed-method:find-method)

(defun displayed-method:get-similar-method-name-at (where &optional include-method-itself)
  "Get the similar method name around WHERE.
With optional include-method-itself, also allow the method title at the top of the buffer.
Works from a displayed method buffer."
  (save-excursion
    (goto-char where)
    (beginning-of-line 1)
    (if (or (looking-at "^ *\\([A-Z0-9 ]+\\): [-&+0-9ETA-D, ]")
	    (looking-at "^ *\\([A-Z0-9 ]+\\) \\(above\\)\\|\\(below\\) the treble")
	    (and include-method-itself
		 (looking-at "^ *\\([A-Z0-9 ]+\\):$")))
	(buffer-matched 1)
      nil)))

(defun displayed-method:display-similar-method (where)
  "From a displayed method buffer, display one of the methods shown as similar."
  (interactive "d")
  (let ((similar-method-name (displayed-method:get-similar-method-name-at where)))
    (if similar-method-name
	(mSiRiL-mode:lines-for-method
	 (ringing:find-named-method similar-method-name))
      (error "Not on a \"similar method\" line"))))

(defun displayed-method:find-method (where)
  "Find the library definition one of the methods shown as similar, or the method itself.
Works from a displayed method buffer."
  (interactive "d")
  (let ((method-name (displayed-method:get-similar-method-name-at where t)))
    (if method-name
	(method-library:locate-method method-name)
      (error "Point not on a method name"))))
		    
(defun displayed-method-mode ()
  "A simple mode for the output of method-displaying code.
Special commands available are:
\\{displayed-method-keymap}
"
  (interactive)
  (kill-all-local-variables)
  (mapcar 'make-local-variable
	  '(displayed-method:method-name))
  (setq major-mode 'displayed-method-mode
	mode-name "Displayed Method"
	;; mode-line-format (copy-sequence mode-line-format)
	;; buffer-read-only t
	)
  (use-local-map displayed-method-keymap)
  (when nil
    (let ((mode-tail (memq 'mode-name mode-line-format)))
      (when mode-tail
	(rplacd mode-tail
		(append '(" " displayed-method:method-name)
			(cdr mode-tail)))))))

(defvar mSiRiL-mode:list-similar-methods nil
  "*Whether to list methods similar to the one just shown.
This requires parsing of the whole file the first time it is done.")

(defvar mSiRiL-mode:identify-above-below nil
  "*Whether to try to describe the method in terms of methods above and below the treble.")

(defun mSiRiL-mode:lines-for-method (method &optional marked-bell-number show-buffer-function)
  "Display the lines for METHOD."
  (let* ((mSiRiL-file (if (change-ringing-method-defining-file method)
			  (change-ringing-method-defining-file method)
			(if (eq major-mode 'mSiRiL-mode)
			    (buffer-file-name)
			  nil)))
	 (nbells (change-ringing-method-nbells method))
	 (method-name (ringing:method-name method))
	 (ringing:observation-number (mSiRiL-mode:choose-bell-to-mark method-name marked-bell-number))
	 (ringing:observation (ringing:bell-number-to-char ringing:observation-number))
	 (ringing:end-of-row-char ? )	; before making the rounds
	 ;; (the-rounds (ringing:make-rounds nbells))
	 (method-display:block-top-marker (make-marker))
	 (place-marker (list 'work)))
    (message "Method %s is from file %s" method-name mSiRiL-file)
    (require 'ermm)
    (ermm:init nbells)
    (funcall (if (null show-buffer-function)
		 'switch-to-buffer-other-window
	       show-buffer-function)
	     (ringing:get-buffer-create (format "*%s*" method-name)))
    (displayed-method-mode)
    (setq displayed-method:method-name method-name)
    (message mSiRiL-mode:ringing-in-progress-format method-name)
    (erase-buffer)
    (insert (format mSiRiL-mode:method-header-format
		    (capitalize method-name)
		    (place-notation:recreate-string-from-changes (change-ringing-method-plain method) t)
		    (place-notation:create-fully-expanded-from-changes (change-ringing-method-plain method))
		    (change-ringing-method-original-description-string method))
	    "\n")
    (setq truncate-lines t)
    (set-marker method-display:block-top-marker (point))
    (insert ?\n)
    (when (eq method-display:main-display-function
	      'method-display:multi-column-display-function)
      (method-display:label-column ringing:observation-number (method-display:column-width nbells))
      (insert ?\n)
      )
    ;; (set-marker method-display:block-top-marker (point))
    (ringing:ring-from-rounds method
			      (if (null (change-ringing-method-autocalls method))
				  'ringing:plain-course-helper-function
				'ringing:dixonoid-plain-course-helper-function)
			      (list place-marker)
			      method-display:main-display-function
			      (list place-marker))
    (when mSiRiL-mode:identify-above-below
      (let* ((just-work nil)
	     (above-below (ringing:identify-method-above-and-below method just-work)))
	(when (and (cdr above-below)
		   (not (string-match method-name (ringing:method-name (cdr above-below)))))
	  (goto-char (point-max))
	  (insert (ringing:method-name (cdr above-below)) "\n"))
	(when (and (car above-below)
		   (not (string-match method-name (ringing:method-name (car above-below)))))
	  (goto-char (point-max))
	  (insert (ringing:method-name (car above-below)) "\n")))
      (goto-char (point-max)) (insert "\n"))
    (when (and mSiRiL-mode:list-similar-methods
	       microSiRiL:parse-whole-files
	       (not (null mSiRiL-file)))
      (method-library:insert-similar-methods method
					     (microSiRiL:parse-file-superficially mSiRiL-file)
					     method-name))
    (goto-char (point-min))
    (set-marker method-display:block-top-marker nil)
    (message nil)))

(defun mSiRiL-mode:trim-bell-number (n)
  "Make sure N is suitable for this mSiRiL-mode buffer."
  (if (numberp n)
      (if (zerop n)
	  10
	(if (> n mSiRiL-nbells)
	    mSiRiL-nbells
	  n))
    n))

(defvar mSiRiL-mode:single-window-display t
  "Whether to switch to a single window in the frame when displaying from mSiRiL-mode.")

(defun mSiRiL-mode:method-buffer-display-function (buffer)
  "Function to display the method buffer when shwoing a method."
  (if mSiRiL-mode:single-window-display
      (progn
	(switch-to-buffer buffer)
	(delete-other-windows))
    (switch-to-buffer-other-window buffer)))

(defun mSiRiL-mode:popup-method-lines (where &optional marked)
  "Display the lines for the method at WHERE (interactively, around point).
Optionally (prefix argument interactively) say which bell to mark --
otherwise chosen for you according to symmetry."
  (interactive "d
p")
  (mSiRiL-mode:lines-for-method (mSiRiL-mode:method-at where)
				(mSiRiL-mode:trim-bell-number marked)
				'mSiRiL-mode:method-buffer-display-function)
  (other-window -1))

(defun mSiRiL-mode:popup-method-both-handbell-lines (where &optional marked)
  "Display both the lines for the method at WHERE (interactively, around point).
Optionally (prefix argument interactively) say which bell to mark --
otherwise chosen for you according to symmetry."
  (interactive "d
p")
  (let ((ringing:green-line-too t))
    (mSiRiL-mode:lines-for-method (mSiRiL-mode:method-at where)
				  (mSiRiL-mode:trim-bell-number marked)
				  'mSiRiL-mode:method-buffer-display-function))
  (other-window -1))

(defun mSiRiL-mode:popup-combined-method-lines (below above &optional marked)
  "Display the lines for the method combined from BELOW and ABOVE,
from mark and point respectively, interactively. Optional / Prefix argument
as for mSiRiL-mode:popup-method-lines."
  (interactive
   (list (mark)
	 (point)
	 (if current-prefix-arg
	     (prefix-numeric-value current-prefix-arg)
	   nil)))
  (let ((combined-method (ringing:find-constructed-method
			  (concat (mSiRiL-mode:method-name-at above)
				  "/"
				  (mSiRiL-mode:method-name-at below)))))
    (mSiRiL-mode:lines-for-method
     combined-method
     (mSiRiL-mode:trim-bell-number marked)
     'mSiRiL-mode:method-buffer-display-function))
  (other-window -1))

(defvar mSiRiL-mode:default-manual-start-at-tenor t
  "*Whether mSiRiL-mode:ring-method's starting point should default to the tenor
or to the lowest bell making places at the lead end.")

(defvar mSiRiL-mode:single-window-ringing t
  "Whether to switch to a single window in the frame when ringing from mSiRiL-mode.")

(defun mSiRiL-mode:ring-method (where &optional marked)
  "Ring interactively a plain course of the method at WHERE (interactively, around point).
Optional second argument (prefix argument interactively) gives which bell to ring --
otherwise chosen for you according to symmetry."
  (interactive "d
p")
  (setq marked (mSiRiL-mode:trim-bell-number marked)
	ringing:clashes 0
	ringing:plain-course-row-count 0)
  (let* ((the-method (mSiRiL-mode:method-at where))
	 (ringing:end-of-row-char ? )
	 (nbells (change-ringing-method-nbells the-method))
	 (method-name (ringing:method-name the-method))
	 (ringing:observation-number
	  (if mSiRiL-mode:default-manual-start-at-tenor
	      (if (= marked 1) nbells marked)
	    (mSiRiL-mode:choose-bell-to-mark method-name marked)))
	 ;; (the-rounds (ringing:make-rounds nbells))
	 (ringing:test-user t)
	 (ringing:allow-timeout-this-row nil) ; set to t after one row
	 (method-display:block-top-marker (make-marker))
	 (ringing:observation (ringing:bell-number-to-char ringing:observation-number))
	 (place-marker (list 'work))
	 (ringing:lead-end-callback 'ringing:lead-end-record-clashes)
	 (ringing:lead-end-callback-args (list (cons ringing:clashes ringing:observation-number)))
	 (ringing-buffer (ringing:get-buffer-create
			  (format ringing:method-buffer-name-format
				  method-name))))
    (save-window-excursion
      (if mSiRiL-mode:single-window-ringing
	  (progn
	    (switch-to-buffer ringing-buffer)
	    (delete-other-windows))
	(switch-to-buffer-other-window ringing-buffer))
      (erase-buffer)
      (set-marker method-display:block-top-marker (point))
      (setq ringing:prev-place ringing:observation-number)
      (if (eq method-display:main-display-function 'method-display:multi-column-display-function)
	  (progn
	    (insert ?\n)
	    (when ringing:label-columns
	      (insert (centre-string-in-width (format "%d" ringing:observation-number)
					      (length (method-display:make-lead-end-line nbells)))
		      ?\n)
	      (end-of-line 1)))
	(progn
	  ))
      (let* ((start-time (current-time))
	     (ringing-result (ringing:ring-from-rounds the-method
						       'ringing:plain-course-helper-function
						       (list place-marker)
						       method-display:main-display-function
						       (list place-marker)))
	     (end-time (current-time))
	     (time-taken (seconds-between start-time end-time)))
	(set-marker method-display:block-top-marker nil)
	(other-window -1)
	(if (eq ringing-result 'user-stand)
	    (progn
	      (message "Touch abandoned"))
	  (progn
	    (touch-book:record-clash-level method-name
					   (cdar ringing:lead-end-callback-args)
					   (- ringing:clashes (caar ringing:lead-end-callback-args)))
	    (message "Touch completed with %d clashes, in %d seconds (peal speed %s)"
		     ringing:clashes time-taken
		     (time-string (* (float time-taken)
				     (/ 5000.0
					(float ringing:plain-course-row-count)))))))))))

(defun mSiRiL-mode:handbells-ring-method (where &optional marked)
  "Ring on handbells a plain course of the method at WHERE (interactively, around point).
Optional second argument (prefix argument interactively) gives which bell to ring --
otherwise chosen for you according to symmetry. The neighbouring bell is also rung."
  (interactive "d
p")
  (require 'handbells)
  (setq marked (mSiRiL-mode:trim-bell-number marked))
  (let* ((the-method (mSiRiL-mode:method-at where))
	 (ringing:end-of-row-char ? )
	 (nbells (change-ringing-method-nbells the-method))
	 (method-name (ringing:method-name the-method)))
    (handbells:ring-method method-name (handbells:right-hand-bell
					(if mSiRiL-mode:default-manual-start-at-tenor
					    (if (= marked 1) nbells marked)
					  (mSiRiL-mode:choose-bell-to-mark method-name marked))))))

(defun mSiRiL-mode:teach-random-lead-of-method (where)
  "Teach a random lead of method given around WHERE (point interactively) in mSiRiL file.
Uses ringing:teach-random-lead-of-method, which see."
  (interactive "d")
  (let ((teach-ringing:allow-timeout-correction-while-teaching t)
	(ringing:use-timing-here t))
    (ringing:teach-random-lead-of-method (mSiRiL-mode:method-at where))))

(defun mSiRiL-mode:teach-random-leads-of-method (where)
  "Teach random leads of method given around WHERE (point interactively) in mSiRiL file,
until the user stands (using up-arrow).
Uses ringing:teach-random-leads-of-method, which see."
  (interactive "d")
  (let ((teach-ringing:allow-timeout-correction-while-teaching t)
	(ringing:use-timing-here t))
    (ringing:teach-random-leads-of-method (mSiRiL-mode:method-at where))))

;;;; intensive teaching

(defun mSiRiL-mode:intensive-teaching (where &optional prefix)
  "Teach the user to ring the method around WHERE (point interactively) in mSiRiL file.
See ringing:teach-each-lead-of-method-until-perfect for further details.
Handle optional prefix arg as for ringing:teach-each-lead-of-method-until-perfect."
  (interactive "d
P")
  (save-window-excursion
    (ringing:teach-each-lead-of-method-until-perfect (mSiRiL-mode:method-at where)
						     (if prefix
							 (prefix-numeric-value prefix)
						       2))))

;;;; interface to editor / explorer

(defun mSiRiL-mode:full-grid-edit (where)
  "Edit the method on the current line."
  (interactive "d")
  (let* ((the-method (mSiRiL-mode:method-at where))
	 (method-pn (place-notation:recreate-string-from-changes
		     (change-ringing-method-plain the-method) t)))
    (require 'full-grid-mode)
    (full-grid-edit mSiRiL-nbells method-pn)))

(defun mSiRiL-mode:full-grid-explore (where)
  "Explore the method on the current line."
  (interactive "d")
  (let* ((the-method (mSiRiL-mode:method-at where))
	 (method-pn (place-notation:recreate-string-from-changes
		     (change-ringing-method-plain the-method) t)))
    (require 'full-grid-mode)
    (full-grid-explore mSiRiL-nbells method-pn)))

;;;; ephemeral method group

(defun mSiRiL-mode:toggle-method-mark (where)
  (interactive "d")
  "Toggle whether the method at WHERE (point, interactively) is in this buffers' ephemeral group."
  (let* ((method-name (mSiRiL-mode:method-name-at where))
	 (marking-pair (assoc method-name mSiRiL-ephemeral-group))
	 (line-start (save-excursion (beginning-of-line 1) (point)))
	 (line-end (save-excursion (end-of-line 1) (point)))
	 (buffer-was-read-only buffer-read-only))
    (when buffer-was-read-only (toggle-read-only -1))
    (if marking-pair
	(progn				; already in, take out
	  (setq mSiRiL-ephemeral-group (delete-if (function (lambda (elt) (string= (car elt) method-name))) mSiRiL-ephemeral-group))
          (mSiRiL-mode:color-line nil (point-max))
	  ;; (put-text-property line-start line-end 'face ringing:default-face)
	  )
      (progn				; out, put in
	(push (cons method-name (mSiRiL-mode:method-at where)) mSiRiL-ephemeral-group)
	(mSiRiL-mode:color-line t (point-max))
	;; (put-text-property line-start line-end 'face ringing:bad-face)
	))
    (when buffer-was-read-only (toggle-read-only 1)))
  (set-buffer-modified-p nil))

(defun mSiRiL-mode:clear-method-marks ()
  (interactive)
  "Clear this buffer's ephemeral group."
  (let ((buffer-read-only nil)
	(case-fold-search t))
    (save-excursion
      (save-restriction
	(widen)
	(goto-char (point-min))
	(while mSiRiL-ephemeral-group
	  (goto-char (point-min))	; they can come in any order
	  (re-search-forward (format "^%s .+$"
				     (microSiRiL:name-used-in-file (caar mSiRiL-ephemeral-group))))
	  ;; (put-text-property (match-beginning 0) (match-end 0) 'face ringing:default-face)
	  (mSiRiL-mode:color-line nil (point-max))
	  (setq mSiRiL-ephemeral-group (cdr mSiRiL-ephemeral-group)))))))

(defun mSiRiL-mode:buffer-ephemeral-group (buffer)
  "Return the ephemeral group for BUFFER (consed with a title), or nil if none."
  (when (stringp buffer) (setq buffer (get-buffer buffer)))
  (let* ((locals (buffer-local-variables buffer))
	 (local-group-pair (assoc 'mSiRiL-ephemeral-group locals)))
    (if (and local-group-pair (cdr local-group-pair))
	(let ((title (format "*Select %s %s methods*"
			     (cdr (assoc 'mSiRiL-method-kind-name locals))
			     (cdr (assoc 'mSiRiL-stage-name locals)))))
	  (cons title
		(make-method-groups:method-group
		 :title title
		 :methods (cdr local-group-pair)
		 :stage (cdr (assoc 'mSiRiL-nbells locals)))))
      nil)))

(defun mSiRiL-mode:buffer-group (buffer)
  "Return a method group containing all the methods of BUFFER, consed with a title."
  (when (stringp buffer) (setq buffer (get-buffer buffer)))
  (save-excursion
    (set-buffer buffer)
    (let ((title (format "*All %s %s methods*"
			 mSiRiL-method-kind mSiRiL-stage-name))
	  (methods (method-library:map-methods-in-file
		    (buffer-file-name buffer)
		    (function (lambda (method)
				(cons (change-ringing-method-title method) method))))))
      (cons title
	    (make-method-groups:method-group
	     :title title
	     :methods methods
	     :stage mSiRiL-nbells)))))

;;;; find other methods with same changes as given one

(defun mSiRiL-mode:methods-with-same-changes (where)
  "Find other methods in this buffer with same changes as the one at WHERE (point, interactively)."
  (interactive "d")
  (let* ((the-method (mSiRiL-mode:method-at where))
	 (this-method-lead-changes (ringing:method-lead-changes the-method))
	 (file-methods-lead-changes (method-library:map-methods-in-file
				     (buffer-file-name)
				     (function
				      (lambda (method)
					(cons method
					      (ringing:method-lead-changes method))))))
	 (matching nil))
    (dolist (possible file-methods-lead-changes)
      (if (string= this-method-lead-changes (cdr possible))
	  (push (ringing:method-name (car possible)) matching)))
    (setq matching (sort matching 'string<))
    (with-output-to-temp-buffer (format "*%s %s methods with same changes as %s*"
					mSiRiL-method-kind-name mSiRiL-stage-name
					(ringing:method-name the-method))
      (dolist (match matching)
	(princ match)
	(princ "\n")))
    matching))

;;;; interface to PostScript method printer

(defvar mSiRiL-mode:ps-temp-files-directory "/tmp/"
  "Directory in which to put the PostScript files for methods.")

(defun ringing:name-as-file-name (name)
  "Convert method-NAME to something more likely to be OK as a filename"
  (let ((filename (copy-sequence name)))
    (dotimes (i (length filename))
      (when (memq (aref filename i) '(? ?*))
	(aset filename i ?_)))
    filename))

(defun ringing:localize-file-name (filename)
  "Return FILENAME in a form that this machine should like."
  (let ((newfilename (copy-sequence filename)))
    (dotimes (i (length newfilename))
      (case system-type
	(windows-nt
	 (when (= (aref newfilename i) ?/)
	   (aset newfilename i ?\\)))))
    newfilename))

(defun mSiRiL-mode:print (where marked-bell
				&optional
				style style-parameters
				header-style header-style-parameters
				grid-style grid-style-parameters)
  "Print the method on the current line (via PostScript)."
  (interactive
   (progn
     (require 'ps-line)
     (let* ((params (ringing:get-postscript-parameters-interactively)))
       (append (list (point) (prefix-numeric-value current-prefix-arg)) params))))
  (mSiRiL-mode:print-or-preview
   where marked-bell
   'ringing:send-postscript-to-printer
   style style-parameters
   header-style header-style-parameters
   grid-style grid-style-parameters))

(defun mSiRiL-mode:preview (where marked-bell
				  &optional
				  style style-parameters
				  header-style header-style-parameters
				  grid-style grid-style-parameters)
  "Preview the method on the current line (via PostScript)."
  (interactive
   (progn
     (require 'ps-line)
     (let* ((params (ringing:get-postscript-parameters-interactively)))
       (append (list (point) (prefix-numeric-value current-prefix-arg)) params))))
  (mSiRiL-mode:print-or-preview
   where marked-bell
   'ringing:send-postscript-to-previewer
   style style-parameters
   header-style header-style-parameters
   grid-style grid-style-parameters))

(defun mSiRiL-mode:print-or-preview (where marked-bell
					   output-function
					   &optional
					   style style-parameters
					   header-style header-style-parameters
					   grid-style grid-style-parameters)
  "Print or preview the method on the current line (via PostScript).
Called from the print and preview commands, which may be used interactively."
  (save-window-excursion
    (let* ((method (mSiRiL-mode:method-at where))
	   (method-name (ringing:method-name method))
	   (ringing:observation (if (= marked-bell 1)
				    (if (integerp ringing:observation)
					ringing:observation
				      ?1)
				  (ringing:bell-number-to-char marked-bell)))
	   (ps-file-name (expand-file-name (format "%s.ps"
						   (ringing:name-as-file-name method-name))
					   mSiRiL-mode:ps-temp-files-directory)))
      (find-file ps-file-name)
      (ringing:postscript-for-method method (current-buffer)
				     style style-parameters
				     header-style header-style-parameters
				     grid-style grid-style-parameters)
      (basic-save-buffer)
      (funcall output-function ps-file-name)
      (unless ringing:keep-postscript-buffers (kill-buffer nil))
      (unless ringing:keep-postscript-files (delete-file ps-file-name)))))

(defun mSiRiL-mode:insert-description (method)
  "Insert a description of METHOD at point."
  (let ((name (ringing:method-name method))
	(code 'y)
	(place-notation (place-notation:recreate-string-from-changes
			 (change-ringing-method-plain method) t t)))
    (insert (format "%s %s %s\n" name code place-notation))))

;;;; syntax colouring

(defun mSiRiL-mode:color-line (marked upto)
  "Color in the current line of the buffer.
If MARKED, highlight it. Don't go beyond UPTO.
Return whether the line contained anything to color."
  (beginning-of-line 1)
  (when (fboundp 'copy-face)
    (if (re-search-forward
	 microSiRiL:line-parts-regexp
	 upto t)
	(let ((red (ringing:get-face marked nil ringing:red))
	      (green (ringing:get-face marked nil ringing:green))
	      (blue (ringing:get-face marked nil ringing:blue)))
	  (put-text-property (match-beginning 1) (match-beginning 2)
			     'face green)
	  (put-text-property (match-beginning 2) (match-beginning 3)
			     'face red)
	  (put-text-property (match-beginning 3) (match-end 3)
			     'face blue)
	  t)
      nil)))

(defun mSiRiL-mode:color-buffer ()
  "Color the current buffer."
  (interactive)
  (mSiRiL-mode:color-buffer-region (point-min) (point-max)))

(defun mSiRiL-mode:color-buffer-region (start end)
  "Color the current buffer between START and END."
  (interactive "r")
  (when (fboundp 'copy-face)
    (let ((buffer-read-only nil)
	  (already-modified (buffer-modified-p))
	  (red (ringing:get-face nil nil ringing:red))
	  (green (ringing:get-face nil nil ringing:green))
	  (blue (ringing:get-face nil nil ringing:blue)))
      (message "Coloring... ")
      (save-excursion
	(goto-char start)
	(while (re-search-forward microSiRiL:line-parts-regexp
				  end t)
	  (put-text-property (match-beginning 1) (match-beginning 2)
			     'face green)
	  (put-text-property (match-beginning 2) (match-beginning 3)
			     'face red)
	  (put-text-property (match-beginning 3) (match-end 3)
			     'face blue)
	  ;; (next-line 1)
	  ))
      (set-buffer-modified-p already-modified)
      (message "Coloring... done"))))
  
(defun mSiRiL-mode:color-visible-area (window window-display-start)
  "Coloring for mSiRiL-mode suitable for use in window-scroll-functions, for
lazy coloring."
  (interactive (list (get-buffer-window (current-buffer) nil) (window-start)))
  (when (fboundp 'copy-face)
    (let ((buf (window-buffer window)))
      ;; (message "Colourer: curr=%s buf=%s" (current-buffer) buf)
      ;;  (set-buffer buf)
      (when (eq major-mode 'mSiRiL-mode)
	(let ((buffer-read-only nil)
	      (already-modified (buffer-modified-p))
	      (red (ringing:get-face nil nil ringing:red))
	      (green (ringing:get-face nil nil ringing:green))
	      (blue (ringing:get-face nil nil ringing:blue))
	      (mred (ringing:get-face t nil ringing:red))
	      (mgreen (ringing:get-face t nil ringing:green))
	      (mblue (ringing:get-face t nil ringing:blue)))
	  (save-excursion
	    (goto-char window-display-start)
	    (vertical-motion (window-height window))
	    (while (and (>= (point) window-display-start)
			(> (point) (point-min)))
	      (when (and (null (get-text-property (point) 'face))
			 (re-search-forward
			  microSiRiL:line-parts-regexp
			  (point-max) t))
		(let* ((method-name (buffer-matched 1))
		       (marked (assoc method-name mSiRiL-ephemeral-group)))
		  (put-text-property (match-beginning 1) (match-beginning 2)
				     'face (if marked mgreen green))
		  (put-text-property (match-beginning 2) (match-beginning 3)
				     'face (if marked mred red))
		  (put-text-property (match-beginning 3) (match-end 3)
				     'face (if marked mblue blue))))
	      (beginning-of-line 0)))
	  (set-buffer-modified-p already-modified))))))

(defun mSiRiL-mode:start-lazy-coloring (&optional stop)
  "Enable lazy coloring in this buffer. With optional (prefix) arg, disable it."
  (interactive "P")
  (when (fboundp 'copy-face)
    (if stop
	(setq window-scroll-functions
	      (delete* 'mSiRiL-mode:color-visible-area window-scroll-functions :test 'eq))
      (when (eq major-mode 'mSiRiL-mode)
	(pushnew 'mSiRiL-mode:color-visible-area window-scroll-functions)
	(mSiRiL-mode:color-visible-area (get-buffer-window (current-buffer) nil) (window-start))))))

(run-hooks 'mSiRiL-mode-load-hook)

;;; end of mSiRiL-mode.el
