;;;; mini-ropes.el -- part of John's elisp change-ringing system
;;; draw little bell-ropes
;;; Time-stamp: <97/06/26 18:59:34 john>

(provide 'mini-ropes)

(defvar mini-ropes:buffer nil
  "Buffer for the mini-ropes.")

(defvar mini-ropes:backstrokes nil
  "Which ropes are at backstroke.")

(defun mini-ropes:init (n)
  "Set up the miniature bell-ropes."
  (interactive "nNumber of ropes: ")
  (setq mini-ropes:buffer (ringing:get-buffer-create "*mini-ropes*")
	mini-ropes:backstrokes (make-vector n nil))
    (split-window nil 6)
    (switch-to-buffer mini-ropes:buffer)
    (erase-buffer)
    (let ((i 0))
      (while (< i n)
	(goto-char (point-min))
	(end-of-line) (insert "  |") (if (eobp) (insert ?\n))
	(end-of-line 2) (insert "  |")	(if (eobp) (insert ?\n))
	(end-of-line 2) (insert "  #") (if (eobp) (insert ?\n))
	(incf i)))
    (other-window 1)
    )

(defun mini-ropes:charpos (bell row)
  "Return the index into the buffer for BELL at ROW."
  (+ (* (1+ (* 3 (length mini-ropes:backstrokes))) row)
     (* bell 3)))

(defun set-buffer-char (where char)
  "Set WHERE of the current buffer to CHAR."
  (delete-region where (1+ where))
  (goto-char where)
  (insert char))

(defvar mini-ropes:input-one-bell-delay .125
  "How long to wait between drawing mini-ropes for input.")

(defun mini-ropes:ring (bell)
  "Ring BELL."
  (save-window-excursion
    (set-buffer mini-ropes:buffer)
    (let ((i (1- bell)))
      (if (aref mini-ropes:backstrokes i)
	  (progn
	    (set-buffer-char (mini-ropes:charpos bell 1) ?|)
	    (set-buffer-char (mini-ropes:charpos bell 2) ?#)
	    (aset mini-ropes:backstrokes i nil))
	(progn
	  (set-buffer-char (mini-ropes:charpos bell 1) ?#)
	  (set-buffer-char (mini-ropes:charpos bell 2) ?|)
	  (aset mini-ropes:backstrokes i t))))))

;;; end of mini-ropes.el
