;;;; ps-config.el -- part of John's elisp change-ringing system
;;; Time-stamp: <98/04/28 12:37:49 john>

(provide 'ps-config)

(defun ringing:postscript-debug (off)
  "Enable (or with prefix arg, disable) debug in the PostScript output."
  (interactive "P")
  (setq ringing:postscript-debug (not off))
  (when ringing:postscript-debug
    (with-output-to-temp-buffer "*Warning*"
      (princ "PostScript now produced will take a long time to print."))))

(defvar ringing:postscript-debug nil
  "Whether to enable debug in the PostScript output.")

(defvar ringing:postscript-font-size 10
  "*The default font size for printing methods in PostScript.
The PostScript code may scale this to fit methods in the space available.")

(defvar ringing:postscript-leading-factor 1.2
  "*How much space to allow between rows, as a multiplier for the font size.")

(defvar ringing:postscript-handstroke-font "Courier"
  "*The font to use for handstrokes when printing methods in PostScript.")

(defvar ringing:postscript-backstroke-font "Courier-Oblique"
  "*The font to use for backstrokes when printing methods in PostScript.")

(defvar ringing:postscript-red-line-width 0.5
  "*The thickness of the red line in PostScript output.")

(defvar ringing:postscript-blue-line-width 1.0
  "*The thickness of the blue line in PostScript output.")

(defvar ringing:postscript-green-line-width 0.75
  "*The thickness of the green line in PostScript output.")

(defvar ringing:postscript-grid-dot-size 0.5
  "*The diameter of the grid dots, if they are used.")

(defun ringing:postscript-write-configuration-dictionary ()
  "Write the configuration dictionary."
  (insert "\n% configuration data:\n/ringingparameters <<\n")
  (insert (format "  /RingingPSdebug %s\n" (if ringing:postscript-debug "true" "false")))
  (insert (format "  /fontsize %f\n" ringing:postscript-font-size))
  (insert (format "  /leading %f %f mul\n" ringing:postscript-font-size ringing:postscript-leading-factor))
  (insert (format "  /handstrokefont /%s findfont %f scalefont\n" ringing:postscript-handstroke-font ringing:postscript-font-size))
  (insert (format "  /backstrokefont /%s findfont %f scalefont\n" ringing:postscript-backstroke-font ringing:postscript-font-size))
  (insert (format "  /redlinewidth %f\n" ringing:postscript-red-line-width))
  (insert (format "  /bluelinewidth %f\n" ringing:postscript-blue-line-width))
  (insert (format "  /greenlinewidth %f\n" ringing:postscript-green-line-width))
  (insert (format "  /griddotsize %f\n" ringing:postscript-grid-dot-size))
  (insert ">> def\n"))

;;; end of ps-config.el
