;; ringing-cus.el - Nice interface to customise John's Emacs Ringing Code.

(provide 'ringing-cus)
(require 'custom)
(require 'ringing)
(require 'ps-line)

(defvar ringing-custom-local-type-properties nil
  "`custom-local-type-properties' for ringing customisation.")
(setq ringing-custom-local-type-properties
      '((number (type . sexp)
                (width . 20)
                (valid . (lambda (c d) (numberp d))))
        (int-or-nil (type . sexp)
                    (width . 20)
                    (valid . (lambda (c d) (or (integerp d) (null d)))))
        (num-or-nil (type . sexp)
                    (width . 20)
                    (valid . (lambda (c d) (or (numberp d) (null d)))))
        (char (type . string)
              (width . 1)
              (valid . (lambda (c d) (and (stringp d) (eq (length d) 1))))
              (import . custom-char-import)
              (export . custom-char-export))
        ))

(defun custom-char-import (custom value)
  (list (format "%c" value)))
(defun custom-char-export (custom value)
  (string-to-char value))

;;(defun custom-char-read (custom string)
;;  "Read from CUSTOM an STRING. Based on `custom-sexp-read' which has a bug in it."
;;  (save-match-data
;;    (let ((cust-pad (custom-padding custom)))
;;      (save-excursion
;;        (set-buffer (get-buffer-create " *Custom Scratch*"))
;;        (erase-buffer)
;;        (insert string)
;;        (goto-char (point-min))
;;        (prog1 (read (current-buffer))
;;          (or (looking-at
;;               (concat (regexp-quote (char-to-string cust-pad))
;;                       "*\\'"))
;;              (error "Junk at end of expression")))))))

(defun ringing:customise ()
  (interactive)
  (switch-to-buffer-other-window (current-buffer))
  ;; reset data before next try...
  (let ((ringing-custom-data
         '((tag . "Emacs Ringing")
           (type . group)
           (doc . "
John's Bell Ringer's Emacs Companion.

This code will enable change ringers to find, print and learn
methods.
")
           (data ((header . nil)
                  (compact . t)
                  (type . group)
                  (doc . "
Press \[Save\] to save any changes permanently after you are done editing.
\[Apply\] allows changes to take effect without saving. \[Reset\] discards
any changes made since last `apply' or `save'.")
                  (data ((tag . "Save")
                         (type . button)
                         (query . ringing:save-variables-cus))
                        ((tag . "Apply")
                         (type . button)
                         (query . custom-apply-all))
                        ((tag . "Reset")
                         (type . button)
                         (query . custom-reset-all))
                        ))
                 ((type . group)
                  (tag . "Viewing Configuration.")
                  (data ((tag . "Single Window When Viewing")
                         (name . mSiRiL-mode:single-window-display)
                         (default . t)
                         (type . toggle))
                        ((tag . "Show Numbers")
                         (name . ringing:show-numbers)
                         (default . t)
                         (type . toggle))
                        ((tag . "Show Blue Line")
                         (name . ringing:show-lines)
                         (default . t)
                         (type . toggle))
                        ((tag . "Show Line Segments")
                         (name . ringing:show-lines-with-lines)
                         (default . t)
                         (type . toggle))
                        ((tag . "Asterisk For Manual Bell")
                         (name . method-display:asterisk-manual-bell)
                         (default . t)
                         (type . toggle))
                        ((tag . "Second line for handbells")
                         (name . ringing:green-line-too)
                         (default . nil)
                         (type . toggle))
                        ((tag . "Show only course bells in numbers display")
                         (name . ringing:display-course-bells-only)
                         (default . nil)
                         (type . toggle))
                        ((tag . "Mark backstrokes")
                         (name . ringing:mark-backstroke)
                         (default . nil)
                         (type . toggle))
                        ((tag . "Show Lead End Twice")
                         (name . method-display:draw-lead-twice)
                         (default . t)
                         (type . toggle))
                        ((tag . "Show Final Rounds")
                         (name . method-display:show-final-rounds)
                         (default . nil)
                         (type . toggle))
                        ((tag . "Multi-Column Format")
                         (name . method-display:main-display-function)
                         (doc . "Toggle the displaying of text in Emacs buffers between multiple
and single column format. You may also specify your own function.")
                         (default . 'method-display:multi-column-display-function)
                         (type . choice)
                         (data ((type . const)
                                (tag . "On ")
                                (default . method-display:multi-column-display-function))
                               ((type . const)
                                (tag . "Off")
                                (default . method-display:single-column-display-function))
                               ((prompt . "Other")
                                (doc . "You must specify the name of a Lisp function here.")
                                (default . __uninitialized__)
                                (type . symbol)
                                (width . 60))
                               ))
			((tag . "Label columns and teaching leads")
			 (name . ringing:label-columns)
			 (default . t)
			 (type . toggle))
                        ((tag . "List Similar Methods")
                         (name . mSiRiL-mode:list-similar-methods)
                         (default . nil)
                         (type . toggle))
			((tag . "Identify methods above and below treble")
                         (name . mSiRiL-mode:identify-above-below)
                         (default . nil)
                         (type . toggle))
                        ((tag . "Mode Highlighting")
                         (name . mSiRiL:do-own-coloring)
                         (default . t)
                         (type . toggle))
                        ((tag . "Mini Ropes")
                         (name . ermm:mini-ropes)
                         (default . nil)
                         (type . toggle))
                        ((tag . "Animation Delay")
                         (name . ringing:auto-ring-inter-row-delay)
                         (default . nil)
                         (type . num-or-nil))
                        ((type . group)
                         (tag . "Printing Characters")
                         (data ((tag . "Treble")
                                (name . ringing:row-lines-treble)
                                (default . ?.)
                                (type . char))
                               ((tag . "Marked Bell")
                                (name . ringing:row-lines-marked-bell)
                                (default . ?*)
                                (type . char))
                               ((tag . "Other Marked Bell")
                                (name . ringing:row-lines-other-marked-bell)
                                (default . ?#)
                                (type . char))
                               ((tag . "Other Bells")
                                (name . ringing:row-lines-other-bells)
                                (default . ? )
                                (type . char))
                               ((tag . "Row Start")
                                (name . ringing:row-lines-start-char)
                                (default . ? )
                                (type . char))
                               ((tag . "Row End")
                                (name . ringing:row-lines-end-char)
                                (default . ? )
                                (type . char))
                               ))
                        ))
                 ((type . group)
                  (tag . "Ringing Configuration.")
                  (data ((tag . "Single Window When Ringing")
                         (name . mSiRiL-mode:single-window-ringing)
                         (default . t)
                         (type . toggle))
                        ((tag . "Clash Buffer")
                         (name . ringing:show-clashes-warning-big)
                         (default . t)
                         (type . toggle))
                        ((tag . "Default to ringing tenor")
                         (name . mSiRiL-mode:default-manual-start-at-tenor)
                         (default . nil)
                         (type . toggle))
                        ((tag . "Highlight Errors")
                         (name . ringing:highlight-bad-rows)
                         (default . t)
                         (type . toggle))
                        ((tag . "Underline Errors")
                         (name . ringing:underline-faults)
                         (default . nil)
                         (type . toggle))
                        ((tag . "Error Off Row Ends")
                         (name . ringing:error-off-end-of-row)
                         (default . nil)
                         (type . toggle))
                        ((tag . "Time Out")
                         (name . ringing:timeout)
                         (default . nil)
                         (type . int-or-nil))
                        ((tag . "Use Timing")
                         (name . ringing:use-timing)
                         (default . nil)
                         (type . toggle))
                        ((tag . "Time Step")
                         (name . ringing:time-step)
                         (default . 0.1)
                         (type . number))
                        ((tag . "Time Early")
                         (name . ringing:time-early)
                         (default . 7)
                         (type . integer))
                        ((tag . "Time Late")
                         (name . ringing:time-late)
                         (default . 13)
                         (type . integer))
                        ))
                 ((type . group)
                  (tag . "Teaching")
                  (data ((tag . "Preview Lead")
                         (name . teach-ringing:preview-leads)
                         (default . t)
                         (type . toggle))
                        ((tag . "Force Different Lead")
                         (name . ringing:never-same-random-consecutive)
                         (default . t)
                         (type . toggle))
                        ((tag . "Number of faults tolerated")
                         (name . teach-ringing:tolerable-faults)
                         (default . 2)
                         (type . integer))
                        ((tag . "Test Previous Leads")
                         (name . teach-ringing:test-previous-leads)
                         (default . nil)
                         (type . toggle))
                        ((tag . "Draw Line Segments When Teaching")
                         (name . ringing:lines-between-rows-when-teaching)
                         (default . t)
                         (type . toggle))
                        ((tag . "Green line for other bell of pair while teaching")
                         (name . ringing:green-line-too-while-teaching)
                         (default . nil)
                         (type . toggle))
                        ((tag . "Number of Perfect Leads With Preview")
                         (name . teach-ringing:required-perfects-seen)
                         (default . 2)
                         (type . integer))
                        ((tag . "Number of Perfect Leads Without Preview")
                         (name . teach-ringing:required-perfects-unseen)
                         (default . 2)
                         (type . integer))
                        ((tag . "Lead Buffer Name Format")
                         (name . ringing:lead-buffer-name-format)
                         (default . "*Place %d of %s*")
                         (type .string))
			((tag . "Time to show shell buffer between quiz leads")
			 (name . ringing:quiz-between-leads-delay-time)
			 (default . 1)
			 (type . integer))
                        ))
                 ((type . group)
                  (tag . "Non Postscript Printing.")
                  (doc . "Control printing of text from Emacs Ringing Buffers.")
                  (data ((tag . "Text Page Width")
                         (name . method-display:page-width)
                         (default . 52)
                         (type . int-or-nil))
                        ((tag . "Form Feed Between Blocks")
                         (name . method-display:form-feed-between-blocks)
                         (default . nil)
                         (type . toggle))
                        ))
                 ((type . group)
                  (tag . "Postscript Options")
                  (doc . "Set up Postscript printing.")
                  (data ((tag . "Postscript Print Command")
                         (name . ringing:ps-print-command)
                         (default . "lpr")
                         (type . string))
                        ((tag . "Font for handstrokes in PostScript printing")
                         (name . ringing:postscript-handstroke-font)
                         (default . "Courier")
                         (type . string))
                        ((tag . "Font for backstrokes in PostScript printing")
                         (name . ringing:postscript-backstroke-font)
                         (default . "Courier-Oblique")
                         (type . string))
                        ((tag . "Font size for PostScript printing")
                         (name . ringing:postscript-font-size)
                         (default . 10)
                         (type . number))
                        ((tag . "Leading factor for PostScript printing")
                         (name . ringing:postscript-leading-factor)
                         (default . 1.2)
                         (type . number))
                        ))
                 ((type . group)
                  (tag . "Files Configuration.")
                  (data ((tag . "Full Progress Messages")
                         (name . method-library:full-progress-messages)
                         (default . t)
                         (type . toggle))
                        ((tag . "Use mSiRiL Libraries")
                         (name . ringing:always-use-method-library)
                         (default . nil)
                         (type . toggle))
                        ((tag . "Always Scan Method libraries")
                         (name . ringing:always-scan-all-libraries)
                         (default . nil)
                         (type . toggle))
                        ((tag . "mSiRiL Format Method Libraries Directory")
                         (name . method-library:directory)
                         (type . file))
                        ((tag . "Ringing World Format Libraries Directory")
                         (name . place-notation:default-methods-directory)
                         (type . file))
                        ((tag . "Groups Directory")
                         (name . method-groups:groups-directory)
                         (type . file))
                        ((tag . "Postscript Temporary Directory")
                         (name . mSiRiL-mode:ps-temp-files-directory)
                         (default . "/tmp/")
                         (type . file))
			((tag . "Include touch book stage groups in group list")
			 (name . method-groups:include-touch-book-stage-groups)
			 (default . nil)
			 (type . toggle))
                        ))
                 ))))
    (let ((custom-local-type-properties ringing-custom-local-type-properties))
      (custom-initialize ringing-custom-data))
    (custom-buffer-create "*Customize*"
                          ringing-custom-data
                          ringing-custom-local-type-properties
                          nil nil
                          'ringing:save-variables-cus)
    (custom-reset-all))
  (search-forward "Emacs Ringing")
  (beginning-of-line)
  (set-window-start (selected-window) (point)))

(defun ringing:save-variables-cus ()
  (interactive)
  (custom-apply-all)
  (ringing:save-variables)
  (kill-buffer (set-buffer "*Customize*")))
