;;; ringing-menu.el -- Addition to John's emacs-ringing lisp

(require 'easymenu)
(require 'mSiRiL-mode)

(autoload 'ringing:customise "ringing-cus" "Customise John's Emacs Ringing System." t)
(autoload 'mSiRiL-mode:custom-print "custom-print" "Pretty interface to printing and previewing." t)

(defvar mSiRiL-menu-items nil
  "The menu for mSiRiL-mode.")

(setq mSiRiL-menu-items
      '(["Show Plain Course"       mSiRiL-mode:popup-method-lines           t]
        ["Above - Below"           mSiRiL-mode:popup-combined-method-lines  t]
        ["List Touch Book"         touch-book:list-my-methods               t]
        ["Worst First"             touch-book:worst-leads-first             t]
        "--"
        ["Ring Plain Course"       mSiRiL-mode:ring-method                  t]
        ["One Random Lead"         mSiRiL-mode:teach-random-lead-of-method  t]
        ["Lots of Random Leads"    mSiRiL-mode:teach-random-leads-of-method t]
        ["Random leads from Group" ringing:teach-random-methods-from-group  t]
        ["Intensive Teaching"      mSiRiL-mode:intensive-teaching           t]
        ["Worst Only"              touch-book:teach-worst-lead              t]
        "--"
        ["Edit Place Notation"     mSiRiL-mode:full-grid-edit           t]
        ["Explore Place Notation"  mSiRiL-mode:full-grid-explore        t]
        ["Print Method"            mSiRiL-mode:print                        t]
        ["Print Group"             method-group:print-group                 t]
        ["Preview Method"          mSiRiL-mode:preview                      t]
        ["Preview Group"           method-group:preview-group               t]
        ["Print/Preview"           mSiRiL-mode:custom-print                 t]
        "--"
        ["Mark/Unmark"             mSiRiL-mode:toggle-method-mark           t]
        "--"
        ("Options"
         ("Viewing"
          ["Single Window Display" (setq mSiRiL-mode:single-window-display
                                         (not mSiRiL-mode:single-window-display))
           :style toggle
           :selected mSiRiL-mode:single-window-display]
          ["Show Numbers" (setq ringing:show-numbers
                                (not ringing:show-numbers))
           :style toggle
           :selected ringing:show-numbers]
          ["Show Lines" (setq ringing:show-lines
                              (not ringing:show-lines))
           :style toggle
           :selected ringing:show-lines]
          ["Show Line Segments" (setq ringing:show-lines-with-lines
                                      (not ringing:show-lines-with-lines))
           :style toggle
           :selected ringing:show-lines-with-lines]
          ["Asterisk" (setq method-display:asterisk-manual-bell
                            (not method-display:asterisk-manual-bell))
           :style toggle
           :selected method-display:asterisk-manual-bell]
          ["2nd Bell" (setq ringing:green-line-too
                            (not ringing:green-line-too))
           :style toggle
           :selected ringing:green-line-too]
          ["Course Bell Only" (setq ringing:display-course-bells-only
                                    (not ringing:display-course-bells-only))
           :style toggle
           :selected ringing:display-course-bells-only]
          ["Mark Backstrokes" (setq ringing:mark-backstroke
                                    (not ringing:mark-backstroke))
           :style toggle
           :selected ringing:mark-backstroke]
          "--"
          "Print Characaters"
          ["Treble"            (ringing:read-char-from-user "Character (in \"lines\") for treble? "            ringing:row-lines-treble)            t]
          ["Marked Bell"       (ringing:read-char-from-user "Character (in \"lines\") for marked bell? "       ringing:row-lines-marked-bell)       t]
          ["Other Marked Bell" (ringing:read-char-from-user "Character (in \"lines\") for other marked bell? " ringing:row-lines-other-marked-bell) t]
          ["Other Bells"       (ringing:read-char-from-user "Character (in \"lines\") for other bells? "       ringing:row-lines-other-bells)       t]
          ["Start of Row"      (ringing:read-char-from-user "Character (in \"lines\") for start of row? "      ringing:row-lines-start-char)        t]
          ["End of Row"        (ringing:read-char-from-user "Character (in \"lines\") for end of row? "        ringing:row-lines-end-char)          t]
          "--"
          ["Draw Lead Twice" (setq method-display:draw-lead-twice
                                   (not method-display:draw-lead-twice))
           :style toggle
           :selected method-display:draw-lead-twice]
          ["Show Final Rounds" (setq method-display:show-final-rounds
                                     (not method-display:show-final-rounds))
           :style toggle
           :selected method-display:show-final-rounds]
          ["Multi-Column" (setq method-display:main-display-function
                                (if (eq method-display:main-display-function
                                        'method-display:single-column-display-function)
                                    'method-display:multi-column-display-function
                                  'method-display:single-column-display-function))
           :style toggle
           :selected (eq method-display:main-display-function 'method-display:multi-column-display-function)]
          ["Label columns and teaching leads" (setq ringing:label-columns
						    (not ringing:label-columns))
	   :style toggle
	   :selected ringing:label-columns]
          ["List Similar Methods" (setq mSiSpeedRiL-mode:list-similar-methods
                                        (not mSiRiL-mode:list-similar-methods))
           :style toggle
           :selected mSiRiL-mode:list-similar-methods]
	  ["Identify methods above and below treble" (setq mSiRiL-mode:identify-above-below
                                        (not mSiRiL-mode:identify-above-below))
           :style toggle
           :selected mSiRiL-mode:identify-above-below]
          "--"
          ["Colour PN Files" (progn
                               (setq mSiRiL:do-own-coloring (not mSiRiL:do-own-coloring))
                               (message "Now revert any place notation files to get colouring."))
           :style toggle
           :selected mSiRiL:do-own-coloring]
          ["Mini Ropes" (setq ermm:mini-ropes
                              (not ermm:mini-ropes))
           :style toggle
           :selected ermm:mini-ropes]
          ["Animation Delay" (setq ringing:auto-ring-inter-row-delay
                             (read-float-from-user "Delay between lines (or nil for fast): " ringing:auto-ring-inter-row-delay t)) t]
          )
         ("Ringing"
          ["Single Window Display" (setq mSiRiL-mode:single-window-ringing
                                         (not mSiRiL-mode:single-window-ringing))
           :style toggle
           :selected mSiRiL-mode:single-window-ringing]
          ["Clash Buffer" (setq ringing:show-clashes-warning-big
                                (not ringing:show-clashes-warning-big))
           :style toggle
           :selected ringing:show-clashes-warning-big]
          ["Default To Tenor" (setq mSiRiL-mode:default-manual-start-at-tenor
                                    (not mSiRiL-mode:default-manual-start-at-tenor))
           :style toggle
           :selected mSiRiL-mode:default-manual-start-at-tenor]
          ["Highlight Errors" (setq ringing:highlight-bad-rows
                                    (not ringing:highlight-bad-rows))
           :style toggle
           :selected ringing:highlight-bad-rows]
          ["Underline Errors" (setq ringing:underline-faults
                                    (not ringing:underline-faults))
           :style toggle
           :selected ringing:underline-faults]
          ["Error Off Row Ends" (setq ringing:error-off-end-of-row
                                      (not ringing:error-off-end-of-row))
           :style toggle
           :selected ringing:error-off-end-of-row]
          ["Timeout" (setq ringing:timeout
                           (read-float-from-user "Timeout: " ringing:timeout t)) t]
          "--"
          ["Use Timing" (setq ringing:use-timing
                              (not ringing:use-timing))
           :style toggle
           :selected ringing:use-timing]
          ["Time Step" (setq ringing:time-step
                             (/
                              (read-float-from-user "Time step (enter time ring a change, in seconds): "
                                                    (* (/ ringing:time-max 2) ringing:time-step))
                              (/ ringing:time-max 2))) t]
          ["Time Early" (setq ringing:time-early
                              (read-int-from-user "End of early time: " ringing:time-early t)) t]
          ["Time Late" (setq ringing:time-late
                             (read-int-from-user "Start of late time: " ringing:time-late t)) t]
          )
         ("Teaching"
          ["Preview" (setq teach-ringing:preview-leads
                           (not teach-ringing:preview-leads))
           :style toggle
           :selected teach-ringing:preview-leads]
          ["Force Different Lead" (setq ringing:never-same-random-consecutive
                                        (not ringing:never-same-random-consecutive))
           :style toggle
           :selected ringing:never-same-random-consecutive]
          ["No. Tolerable Faults" (setq teach-ringing:tolerable-faults
                                        (read-int-from-user
                                         "How many faults are acceptable in intensive learning: "
                                         teach-ringing:tolerable-faults)) t]
          ["Test Previous Leads" (setq teach-ringing:test-previous-leads
                                       (not teach-ringing:test-previous-leads))
           :style toggle
           :selected teach-ringing:test-previous-leads]
          ["Line Segments When Teaching" (setq ringing:lines-between-rows-when-teaching
                                               (not ringing:lines-between-rows-when-teaching))
           :style toggle
           :selected ringing:lines-between-rows-when-teaching]
          ["Show Green Line When Teaching" (setq ringing:green-line-too-while-teaching
                                                 (not ringing:green-line-too-while-teaching))
           :style toggle
           :selected ringing:green-line-too-while-teaching]
          "--"
          "Set No. Perfect Tests"
          ["Seen" (setq teach-ringing:required-perfects-seen
                        (read-int-from-user
                         "Times user must get lead right in intensive learning (teaching phase): "
                         teach-ringing:required-perfects-seen)) t]
          ["Unseen" (setq teach-ringing:required-perfects-unseen
                          (read-int-from-user
                           "Times user must get lead right in intensive learning (teaching phase): "
                           teach-ringing:required-perfects-seen)) t]
	  "--"
	  ["Time for shell in quiz" (setq ringing:quiz-between-leads-delay-time
								(read-int-from-user
								 "Time to show shell buffer between quiz leads: "
								 ringing:quiz-between-leads-delay-time)) t]
          "--"
          ["Lead Buffer Name" (setq ringing:lead-buffer-name-format
                                    (if (y-or-n-p "Name each teaching lead buffer differently: ")
                                        "*Place %d of %s*"
                                      "*One lead*")) t]
          )
         ("Printing"
          "Text"
          ["Set Page Width" (setq method-display:page-width
                                  (read-int-from-user "Page width: " method-display:page-width t)) t]
          ["Form Feed between Columns" (setq method-display:form-feed-between-blocks
                                             (not method-display:form-feed-between-blocks))
           :style toggle
           :selected method-display:form-feed-between-blocks]
          "--"
          "Postscript"
          ["Set Print Command" (setq ringing:ps-print-command
                                     (read-from-minibuffer "Command to send PostScript to printer: " ringing:ps-print-command)) t]
          "--"
          ["Handstroke Font" (setq ringing:postscript-handstroke-font
                                   (read-from-minibuffer "Font for handstrokes in PostScript printing: " ringing:postscript-handstroke-font)) t]
          ["Backstroke Font" (setq ringing:postscript-backstroke-font
                                   (read-from-minibuffer "Font for backstrokes in PostScript printing: " ringing:postscript-backstroke-font)) t]
          ["Font Size" (setq ringing:postscript-font-size
                             (read-float-from-user "Font size for PostScript printing" ringing:postscript-font-size)) t]
          ["Leading Factor" (setq ringing:postscript-leading-factor
                                  (read-float-from-user "Leading factor for PostScript printing" ringing:postscript-leading-factor)) t]
          )
         ("Files"
          ["Progress Messages" (setq method-library:full-progress-messages
                                     (not method-library:full-progress-messages))
           :style toggle
           :selected method-library:full-progress-messages]
          "--"
          ["Use mSiRiL Libs." (setq ringing:always-use-method-library
                                   (not ringing:always-use-method-library))
           :style toggle
           :selected ringing:always-use-method-library]
          ["Always Scan Libs." (setq ringing:always-scan-all-libraries
                                     (not ringing:always-scan-all-libraries))
           :style toggle
           :selected ringing:always-scan-all-libraries]
          "--"
          ["Set mSiRiL Lib. Dir." (setq method-library:directory
                                   (read-directory-name "Method library directory: " method-library:directory)) t]
          ["Set RW Lib. Dir." (setq place-notation:default-methods-directory
                                           (read-directory-name "Personal default methods directory: " place-notation:default-methods-directory)) t]
          ["Set Groups Dir." (setq method-groups:groups-directory
                                   (read-directory-name "Personal default methods directory: " method-groups:groups-directory)) t]
          ["Set Temp. Dir." (setq mSiRiL-mode:ps-temp-files-directory
                                  (read-directory-name "PostScript temporary files directory: " mSiRiL-mode:ps-temp-files-directory)) t]
          ["Include touch book groups" (setq method-groups:include-touch-book-stage-groups
					     (y-or-n-p "Include touch book stage groups in group list? ")) t]
          )
         ["Save setup" ringing:save-variables t]
         )
        ("Methods"
         ("Plain"
          ["Minimus"   (find-file (expand-file-name "p4" method-library:directory)) t]
          ["Doubles"   (find-file (expand-file-name "p5" method-library:directory)) t]
          ["Minor"     (find-file (expand-file-name "p6" method-library:directory)) t]
          ["Triples"   (find-file (expand-file-name "p7" method-library:directory)) t]
          ["Major"     (find-file (expand-file-name "p8" method-library:directory)) t]
          ["Caters"    (find-file (expand-file-name "p9" method-library:directory)) t]
          ["Royal"     (find-file (expand-file-name "p10" method-library:directory)) t]
          ["Cinques"   (find-file (expand-file-name "p11" method-library:directory)) t]
          ["Maximus"   (find-file (expand-file-name "p12" method-library:directory)) t]
          ["Sextuples" (find-file (expand-file-name "p13" method-library:directory)) t]
          ["14"        (find-file (expand-file-name "p14" method-library:directory)) t]
          ["Septuples" (find-file (expand-file-name "p15" method-library:directory)) t]
          ["16"        (find-file (expand-file-name "p16" method-library:directory)) t]
          )
         ("Treble Bob"
          ["Minor"   (find-file (expand-file-name "t6" method-library:directory)) t]
          ["Major"   (find-file (expand-file-name "t8" method-library:directory)) t]
          ["Royal"   (find-file (expand-file-name "t10" method-library:directory)) t]
          ["Maximus" (find-file (expand-file-name "t12" method-library:directory)) t]
          ["14"      (find-file (expand-file-name "t14" method-library:directory)) t]
          ["16"      (find-file (expand-file-name "t16" method-library:directory)) t]
          )
         ("Surpise"
          ["Minor"   (find-file (expand-file-name "s6" method-library:directory)) t]
          ["Major"   (find-file (expand-file-name "s8" method-library:directory)) t]
          ["Royal"   (find-file (expand-file-name "s10" method-library:directory)) t]
          ["Maximus" (find-file (expand-file-name "s12" method-library:directory)) t]
          ["14"      (find-file (expand-file-name "s14" method-library:directory)) t]
          ["16"      (find-file (expand-file-name "s16" method-library:directory)) t]
          )
         ("Delight"
          ["Minor"   (find-file (expand-file-name "d6" method-library:directory)) t]
          ["Major"   (find-file (expand-file-name "d8" method-library:directory)) t]
          ["Royal"   (find-file (expand-file-name "d10" method-library:directory)) t]
          ["Maximus" (find-file (expand-file-name "d12" method-library:directory)) t]
          ["14"      (find-file (expand-file-name "d14" method-library:directory)) t]
          ["16"      (find-file (expand-file-name "d16" method-library:directory)) t]
          )
         ("Alliance"
          ["Doubles"   (find-file (expand-file-name "p5" method-library:directory)) t]
	  ["Minor"     (find-file (expand-file-name "p6" method-library:directory)) t]
	  ["Major"     (find-file (expand-file-name "p8" method-library:directory)) t]
	  ["Royal"     (find-file (expand-file-name "p10" method-library:directory)) t]
	  ["Maximus"   (find-file (expand-file-name "p12" method-library:directory)) t]
	  ["14"        (find-file (expand-file-name "p14" method-library:directory)) t]
	  ["16"        (find-file (expand-file-name "p16" method-library:directory)) t]
	  )
	 ("Treble Place"
	  ["Doubles"   (find-file (expand-file-name "tp5" method-library:directory)) t]
	  ["Minor"     (find-file (expand-file-name "tp6" method-library:directory)) t]
	  ["Triples"   (find-file (expand-file-name "tp7" method-library:directory)) t]
	  ["Major"     (find-file (expand-file-name "tp8" method-library:directory)) t]
	  ["Caters"    (find-file (expand-file-name "tp9" method-library:directory)) t]
	  ["Cinques"   (find-file (expand-file-name "tp11" method-library:directory)) t]
	  )
	 ("Hybrid"
	  ["Minor"     (find-file (expand-file-name "h6" method-library:directory)) t]
	  ["Major"     (find-file (expand-file-name "h8" method-library:directory)) t]
	  ["Royal"     (find-file (expand-file-name "h10" method-library:directory)) t]
	  )
	 ("Principles"
	  ["Minor"     (find-file (expand-file-name "o6" method-library:directory)) t]
	  ["Triples"   (find-file (expand-file-name "o7" method-library:directory)) t]
	  ["Major"     (find-file (expand-file-name "o8" method-library:directory)) t]
	  ["Caters"    (find-file (expand-file-name "o9" method-library:directory)) t]
	  ["Royal"     (find-file (expand-file-name "o10" method-library:directory)) t]
	  ["Cinques"   (find-file (expand-file-name "o11" method-library:directory)) t]
	  ["Maximus"   (find-file (expand-file-name "o12" method-library:directory)) t]
	  ["Sextuples" (find-file (expand-file-name "o13" method-library:directory)) t]
	  ["Septuples" (find-file (expand-file-name "o15" method-library:directory)) t]
	  )
	 )
	"--"
        ["Colour Buffer" mSiRiL-mode:color-buffer t]
        ["Colour Region" mSiRiL-mode:color-buffer-region t]
        ["Customise" ringing:customise t]
	["Tidy Up" ringing:tidy-up t]
	))

(easy-menu-define mSiRiL-menu mSiRiL-keymap
                  "mSiRiL Menu" (cons "mSiRiL" mSiRiL-menu-items))

;; Check to see what is missing from the menu.
(defun check-ringing-menu ()
  "Help to keep the menu up to date with respect to `ringing:set-variables'."
  (interactive)
  (let ((list ringing:set-variables)
        (message-log-max 100)
        var-list)
    (while list
      (setq var-list (cons (car (cdr (car list))) var-list)
            list (cdr list)))
    (setq var-list (nreverse var-list)) ;; Same order as ringing:set-variables as far as possible
    (while var-list
      (beginning-of-buffer)
      (if (search-forward (format "%s" (car var-list)) (point-max) t)
          nil
        (message "%s not found" (car var-list)))
      (setq var-list (cdr var-list)))))

(provide 'ringing-menu)

;; Local Variables:
;; truncate-lines:t
;; End:
