;;;; ringing-print-custom.el -- part of John's elisp change-ringing system
;;; Time-stamp: <98/02/27 14:39:28 john>

;; Provide a dialogue box like interface for printing.

(require 'custom)
(require 'mSiRiL-mode)
(require 'ps-line)
(require 'method-groups)
(require 'stages)

;; John you should be using a global variable like this I think. You
;; can put it in the right place. I don't think it belongs here.

(defvar ringing:keep-postscript-file nil
  "*If non-nil, the postscript file will not be deleted after]
sending the job to the printer.")

(defvar custom-print:selected-method (make-marker)
  "Marker for the line in the msiril file from which
`mSiRiL-mode:custom-print' was invoked.")

(defvar custom-print:group nil
  "If non-nil, the group to be printed, otherwise no group.")

(defvar custom-print:printed-title ""
  "The title to print with a group style.")

;; One entry point for both single method printing and groups.

(defun mSiRiL-mode:custom-print ()
  "Provide a dialogue box like interface for printing."
  (interactive)
  (set-marker custom-print:selected-method (point))
  (switch-to-buffer-other-window (current-buffer))
  ;; reset data before next try...
  (let ((ringing-type-properties
         (append
          (list
           (cons 'group-choose;; What about ephemeral group?
                 (list '(type . choice)
                       (cons 'data (append '(((tag . "None")
                                              (default . nil)
                                              (type . const)))
                                           (mapcar '(lambda (x)
                                                      (list (cons 'tag (car x))
                                                            (cons 'default (car x))
                                                            '(type . const)))
                                                   (method-groups:all-groups))))))

           (cons 'group-style-choose
                 (list '(type . choice)
                       (cons 'data (mapcar '(lambda (x)
                                              (list (cons 'tag (car x))
                                                    (cons 'default (car x))
                                                    '(type . const)))
                                           (ringing:ps-group-styles)))))
           (cons 'title-style-choose
                 (list '(type . choice)
                       (cons 'data (mapcar '(lambda (x)
                                              (list (cons 'tag (car x))
                                                    (cons 'default (car x))
                                                    '(type . const)))
                                           (ringing:ps-title-styles)))))
           (cons 'line-style-choose
                 (list '(type . choice)
                       (cons 'data (mapcar '(lambda (x)
                                              (list (cons 'tag (car x))
                                                    (cons 'default (car x))
                                                    '(type . const)))
                                           (ringing:ps-styles))))))
          '((number (type . sexp)
                    (width . 20)
                    (valid . (lambda (c d) (numberp d))))
            )))
        (ringing-custom-data
         '((tag . "Method Printing in PostScript")
           (type . group)
           (data ((header . nil)
                  (compact . t)
                  (type . group)
                  (doc . "
Press \[Save\] to save any changes permanently after you are done editing.
\[Apply\] allows changes to take effect without saving. \[Reset\] discards
any changes made since last `apply' or `save'.")
                  (data ((tag . "Print")
                         (type . button)
                         (query . mSiRiL-mode:custom-print-button))
                        ((tag . "Apply")
                         (type . button)
                         (query . custom-apply-all))
                        ((tag . "Reset")
                         (type . button)
                         (query . custom-reset-all))
                        ))
                 ((type . group)
                  (tag . "Printing Configuration.")
                  (data ((tag . "Title Style")
                         (name . ringing:latest-ps-title-style)
                         (default . "default")
                         (type . title-style-choose))
                        ((tag . "Blue Line Style")
                         (name . ringing:latest-ps-style)
                         (default . t)
                         (type . line-style-choose))
                        ((tag . "Group")
                         (name . custom-print:group)
                         (default . nil)
                         (type . group-choose))
                        ((tag . "Group Style")
                         (name . ringing:latest-ps-group-style)
                         (default . "default")
                         (type . group-style-choose))
                        ((tag . "Group Title")
                         (name . custom-print:printed-title)
                         (default . "")
                         (type . string))
                        ((tag . "Keep PostScript File")
                         (name . ringing:keep-postscript-file)
                         (default . nil)
                         (type . toggle))
                        ))
                 ((header . nil)
                  (compact . t)
                  (type . group)
                  (data ((tag . "Print")
                         (type . button)
                         (query . mSiRiL-mode:custom-print-button))
                        ((tag . "Apply")
                         (type . button)
                         (query . custom-apply-all))
                        ((tag . "Reset")
                         (type . button)
                         (query . custom-reset-all))
                        ))
                 ))))
    (custom-buffer-create "*Customize*" ringing-custom-data ringing-type-properties)
    (custom-reset-all))
  (search-forward "Method Printing in PostScript")
  (beginning-of-line)
  (set-window-start (selected-window) (point)))

(defun mSiRiL-mode:custom-print-button ()
  "Call back function to actuallt print something."
  (interactive)
  (custom-apply-all)
  (let ((buf (get-buffer "*Customize*")))
    (delete-window (get-buffer-window buf nil))
    (kill-buffer buf))
  (if custom-print:group
      (method-group:print-group (let ((group-stage (method-group:group-stage
                                                    ringing:latest-ps-group-style)))
                                  (if (null group-stage)
                                      (ringing:read-stage-from-user "At stage: ")
                                    (ringing:stage-name group-stage)))
                                custom-print:group
                                ringing:latest-ps-style
                                (ringing:prompt-for-style-parameters
                                 (expand-file-name ringing:latest-ps-style
                                                   ringing:postscript-styles-dir))
                                ringing:latest-ps-title-style
                                (ringing:prompt-for-style-parameters
                                 (expand-file-name ringing:latest-ps-title-style
                                                   ringing:postscript-group-styles-dir))
                                ringing:latest-ps-group-style
                                (ringing:prompt-for-style-parameters
                                 (expand-file-name ringing:latest-ps-group-style
                                                   ringing:postscript-group-styles-dir))
                                custom-print:printed-title)
    (mSiRiL-mode:print custom-print:selected-method
                       ringing:latest-ps-style
                       (ringing:prompt-for-style-parameters
                        (expand-file-name ringing:latest-ps-style
                                          ringing:postscript-styles-dir))
                       ringing:latest-ps-title-style
                       (ringing:prompt-for-style-parameters
                        (expand-file-name ringing:latest-ps-title-style
                                          ringing:postscript-title-styles-dir))))
  (set-marker custom-print:selected-method nil))

;;; end of ringing-print-custom.el
