;;;; teacher.el -- part of John's elisp change-ringing system
;;; teach touches
;;; Time-stamp: <99/10/18 11:45:41 johnstu>

(eval-when (compile load eval)
  (provide 'teacher)
  (require 'user-mistakes)
  (require 'ringing-methods)
  (require 'bell-numbers)
  (require 'leads)
  (require 'beer))

(defun teach-ringing:old-get-user-stroke (row
				      remaining-changes
				      method
				      rounds
				      teaching-info
				      place-marker
				      &optional extra-prompt)
  "This is for calling from callback functions for the ringing system."
  (when (null ringing:prev-place) (setq ringing:prev-place ringing:observation-number))
  (let* ((nbells (change-ringing-method-nbells method))
         (ringing:error-off-end-of-row nil) ; don't let user go off end, simpler for now!
	 (place (ringing:read-number-up-to (if (stringp extra-prompt)
					       (format "%sRing at: " extra-prompt)
					     "Ring at: ")
					   nbells ringing:prev-place))
	 (clashed nil)
	 (timeout-intervened nil))
    (when (eq place 'timeout)
      (message "Correcting learner on timeout")
      (setq place (ringing:what-user-should-have-typed row)
	    clashed t
	    timeout-intervened t))
    (while (and (/= (aref row place)
		    ringing:observation)
		(/= (aref row place)
		    ?*))
      (if (not clashed)
	  ;; first time round, record what the user got wrong
	  (let ((which-row (user-mistakes:lead-which-row teaching-info))
		(all-row-faults (user-mistakes:lead-row-faults teaching-info)))
	    (when (null all-row-faults)
	      (setq all-row-faults (make-vector (* nbells 8) ; should be enough for just about any method!
						nil))
	      (setf (user-mistakes:lead-row-faults teaching-info) all-row-faults))
	    (let* ((row-faults (aref all-row-faults
				     which-row))
		   (just-rung-row (car remaining-changes))
		   (given-next-place (aref just-rung-row ringing:prev-place))
		   (given-direction (- given-next-place ringing:prev-place))
		   (user-direction (- place ringing:prev-place)))
	      (when (null row-faults)
		(setq row-faults (make-vector 4 0))
		(aset row-faults (1+ given-direction) t)
		(aset all-row-faults which-row row-faults))
	      (incf (aref row-faults (1+ user-direction))))
	    (setq clashed t)
	    )
	(progn;; other times, tell them they're guessing
	  (ringing:message "Are you just guessing?")))
      (incf ringing:clashes)
      (setq ringing:bad-row t)
      (unless timeout-intervened
	(let ((error-message 
	       (format "Wrong: you rang bell %c in %d, where %c should be!"
		       ringing:observation
		       place (aref row place))))
	  (if ringing:show-clashes-warning-big
	      (save-window-excursion
		(with-output-to-temp-buffer ringing:clash-message-buffer-name
		  (princ (format "%d clashes so far\n" ringing:clashes))
		  (princ error-message))
		(setq place
		      (ringing:read-number-up-to
		       "Ring at: " nbells ringing:prev-place)))
	    (progn
	      (ringing:message error-message)
	      (setq place
		    (ringing:read-number-up-to
		     "Ring at: " nbells ringing:prev-place)))))))
    ;; (setq footeach teaching-info)
    (incf (user-mistakes:lead-which-row teaching-info))
    (setq ringing:prev-place place)))

(defun teach-ringing:get-user-stroke (row
				      remaining-changes
				      method
				      rounds
				      teaching-info
				      place-marker
				      &optional extra-prompt)
  "This is for calling from callback functions for the ringing system.
It is similar to the normal ringing:get-user-stroke, but records more detail
about mistakes."
  (when (null ringing:prev-place) (setq ringing:prev-place ringing:observation-number))
  (let* ((nbells (change-ringing-method-nbells method))
         (ringing:error-off-end-of-row nil) ; don't let user go off end, simpler for now!
	 (ringing:use-timing-here ringing:allow-timeout-this-row)
	 (place (ringing:read-number-up-to (if (stringp extra-prompt)
					       (format "%sRing at: " extra-prompt)
					     "Ring at: ")
					   nbells ringing:prev-place))
	 (clashed nil)
	 (timeout-intervened nil))
    (when (eq place 'timeout)
      (message "Correcting learner on timeout")
      (setq place (ringing:what-user-should-have-typed row)
	    clashed t
	    timeout-intervened t))
    (while (and (/= (aref row place)
		    ringing:observation)
		(/= (aref row place)
		    ?*))
      (if (not clashed)
	  ;; first time round, record what the user got wrong
	  (progn
	    (user-mistakes:record-mistake teaching-info remaining-changes place nbells)
	    (setq clashed t
		  ringing:auto-hunt-state nil))
	(progn;; other times, tell them they're guessing
	  (ringing:message "Are you just guessing?")))
      (incf ringing:clashes)
      (setq ringing:bad-row t
	    ringing:auto-hunt-state nil)
      (unless timeout-intervened
	(let ((error-message
	       (format "Wrong: you rang bell %c in %d, where %c should be!"
		       ringing:observation
		       place (aref row place))))
	  (if ringing:show-clashes-warning-big
	      (save-window-excursion
		(with-output-to-temp-buffer ringing:clash-message-buffer-name
		  (princ (format "%d clashes so far\n" ringing:clashes))
		  (princ error-message)))
            (ringing:message error-message))
          (setq place
                (ringing:read-number-up-to
                 "Ring at: " nbells ringing:prev-place))
          (when (eq place 'timeout)
            (message "Correcting learner on timeout")
            (setq place (ringing:what-user-should-have-typed row)
                  clashed t
                  timeout-intervened t)))))
    ;; (setq footeach teaching-info)
    (incf (user-mistakes:lead-which-row teaching-info))
    (setq ringing:allow-timeout-this-row t)
    (setq ringing:prev-place place)))

(defvar teach-ringing:observation-last-seen-at nil
  "The place in which the teaching code last saw the observation bell.")

(defun teach-ringing:display-function (row
				       remaining-changes
				       method-holder
				       rounds
				       teaching-info
				       place-marker
				       display-performance)
  "Helper function for ringing plain courses optionally interactively with stander-by."
  (let ((ringing:bad-row nil)
	(method (car method-holder))
	(extra-prompt (if (<= (length remaining-changes) ringing:call-at-remaining-rows)
			  (case (car ringing:next-lead-end-action)
			    (plain nil)
			    (bob "Bob! ")
			    (single "Single! ")
			    (change-of-method (format "%s! " (cadr ringing:next-lead-end-action))))
			nil)))
    (when ringing:test-user
      (teach-ringing:get-user-stroke row
				     remaining-changes
				     method
				     rounds
				     teaching-info
				     place-marker
				     extra-prompt))
    (insert "\n")
    (ringing:apply-bell-colours row)
    (ermm:output-row row)
    (ringing:insert-row-lines row
			      (if display-performance
				  (user-mistakes:performance teaching-info)
				extra-prompt))
    (setq teach-ringing:observation-last-seen-at (ringing:observation-place-number row))))

(defun teach-ringing:display-touch-function (row
					     remaining-changes
					     method-holder
					     rounds
					     place-marker
					     calls
					     truth-holder)
  "Adaptor between touch ringing and teaching."
  (declare '(special method-name-tidy))
  (setq method-name-tidy (capitalize (ringing:method-name (car method-holder))))
  (teach-ringing:display-function row
				  remaining-changes
				  method-holder
				  rounds
				  user-mistakes:teaching-info
				  place-marker
				  nil))

(defvar teach-ringing:in-teach-until-perfect nil
  "Whether we are in \"teach until perfect\" mode -- to tell the help routine.")

(defvar teach-ringing:current-method nil
  "The method currently being taught -- for crib sheet purposes
and perhaps others.")

(defun teach-ringing:extra-help ()
  "Output extra help while teaching -- a crib sheet for the method"
  (when teach-ringing:in-teach-until-perfect
    (princ (format "The program will get you to try each lead of %s.\n"
		   (ringing:method-name teach-ringing:current-method)))
    (princ (format "You are shown the lead until you have got it right %d times (settable\nthrough variable teach-ringing:required-perfects-seen)\n"
		   teach-ringing:required-perfects-seen))
    (princ (format "then have to ring it without being shown it until you have got it right\n%d times (settable through variable teach-ringing:required-perfects-unseen).\n"
		   teach-ringing:required-perfects-unseen))
    (princ (format "\"Getting it right\" means not more than %d mistakes (settable through\nvariable teach-ringing:tolerable-faults).\n"
		   teach-ringing:tolerable-faults)))
  (require 'crib-line)
  (ringing:crib-line-to-stdout teach-ringing:current-method ringing:observation))

(defvar ringing:lead-buffer-name-format
  "*Place %d of %s*"
  "Format string for naming teaching buffers, with PLACE (%d) and NAME (%s) to be substituted.")

(defvar ringing:green-line-too-while-teaching nil
  "Whether to show the green line while teaching.
Having this on is probably mostly a distraction, even if you're usually using
the green line; but you might want it, if learning for handbells.")

(defun teach-ringing:do-one-lead (method-name
				  manual-bell
				  with-user
				  show-performance
				  &optional buffer-selection-function)
  "Ring a lead of METHOD-NAME, with the user on MANUAL-BELL if WITH-USER; or SHOW-PERFORMANCE."
  (interactive
   (let* ((method-name (ringing:read-method-name "Ring lead of: "))
	  (user (y-or-n-p "Ring interactively? "))
	  (manual (ringing:read-number-up-to
		   (if user "Manual bell" "Mark bell")
		   (change-ringing-method-nbells
		    (ringing:find-named-method method-name)))))
     (list method-name manual user)))
  (unless show-performance
    (funcall (if buffer-selection-function buffer-selection-function 'switch-to-buffer)
	     (ringing:get-buffer-create
	      (format ringing:lead-buffer-name-format
		      manual-bell
		      (ringing:method-name method-name))))
    (erase-buffer)
    (when ringing:label-columns
      (insert (format "Place bell %d of %s\n" manual-bell (ringing:method-name method-name))))
    (unless buffer-selection-function (delete-other-windows))
    (setq ringing:clashes 0))
  (setq ringing:prev-place (ringing:bell-char-to-number manual-bell))
  (let* ((ringing:observation-number manual-bell)
	 (ringing:observation (ringing:bell-number-to-char manual-bell))
	 (teach-ringing:current-method (ringing:find-named-method method-name))
	 (ringing:end-of-row-char ? )	; before making the rounds
	 (ringing:test-user with-user)
	 (ringing:allow-timeout-this-row nil) ; set to t after one row
	 (ringing:green-line-too ringing:green-line-too-while-teaching)
	 (ringing:bell-input-extra-help-function (if with-user
						     'teach-ringing:extra-help
						   ringing:bell-input-extra-help-function))
	 (place-marker (list 'work)))
    (require 'ermm)
    (ermm:init (change-ringing-method-nbells teach-ringing:current-method))
    (let ((result (ringing:ring-from-rounds teach-ringing:current-method
					    'ringing:ring-one-lead-helper-function
					    (list place-marker)
					    'teach-ringing:display-function
					    (list user-mistakes:teaching-info place-marker show-performance))))
      (if (eq result 'user-stand)
	  (bury-buffer)
	(when with-user
          (touch-book:record-clash-level (ringing:method-name method-name) manual-bell ringing:clashes)
	  (with-output-to-temp-buffer "*Ringing report*"
	    (princ (format "Lead finished with %d clashes\n" ringing:clashes)))))
      result)))

(defvar teach-ringing:preview-leads nil
  "*Whether to show the user each lead before getting them to try it,
in teaching functions.")

(defvar ringing:previous-random-bell 0
  "Which bell was last used as a randomly chosen bell;
instead of really random, we pick a different bell each time.")

(defvar ringing:never-same-random-consecutive t
  "*Whether to prevent the random bell chooser from presenting the same bell twice running.")

(defun ringing:random-working-bell (method-name)
  "Pick a random working bell from METHOD-NAME."
  (let* ((method (ringing:find-named-method method-name))
	 (base 2)
	 (limit-minus-base (- (change-ringing-method-nbells method) 1))
	 (choice (+ (random limit-minus-base) base)))
    (when ringing:never-same-random-consecutive
      (while (= choice ringing:previous-random-bell)
	(setq choice (+ (random limit-minus-base) base))))
    (setq ringing:previous-random-bell choice)
    choice))

(defun ringing:teach-random-lead-of-method (method-name)
  "Show the user a random lead of METHOD-NAME, then let them try it themselves."
  (interactive
   (list (ringing:read-method-name "Ring random lead of: ")))
  (let ((ringing:use-timing-here t))
    (ringing:teach-lead-of-method method-name (ringing:random-working-bell method-name))))

(defvar ringing:lines-between-rows-when-teaching t
  "Whether to draw lines between the rows when getting the user to try something
during a teaching command.")

(defun ringing:try-user-on-lead (method place-bell)
  "Test the user on one lead of METHOD starting at PLACE-BELL."
  (interactive
   (let* ((method (ringing:read-method-name "Test a lead of: "))
	  (manual-bell (ringing:read-number-up-to
			"Place bell"
			(change-ringing-method-nbells
			 (ringing:find-named-method method)))))
     (list method manual-bell)))
  (message "Try ringing place %d of %s" place-bell (ringing:method-name method))
  (let ((user-mistakes:teaching-info (user-mistakes:get-teaching-data (ringing:find-named-method method) place-bell))
	(ringing:show-lines-with-lines ringing:lines-between-rows-when-teaching))
    (setf (user-mistakes:lead-which-row user-mistakes:teaching-info) 0)
    (incf (user-mistakes:lead-attempts user-mistakes:teaching-info))
    (let ((result
	   (teach-ringing:do-one-lead method place-bell t nil)))
      (setf (user-mistakes:lead-latest-faults user-mistakes:teaching-info) ringing:clashes)
      (incf (user-mistakes:lead-total-faults user-mistakes:teaching-info) ringing:clashes)
      (when (zerop ringing:clashes)
	(incf (user-mistakes:lead-perfects user-mistakes:teaching-info))
	(message "Congratulations -- no clashes"))
      result)))

(defun ringing:try-auto-on-lead (method place-bell)
  "Don't test the user on one lead of METHOD starting at PLACE-BELL.
Instead, ring it automatically. This is a for debugging things that
call ringing:try-user-on-lead without having to ring everything myself!
Arguments and results are in the same form. as ringing:try-user-on-lead."
  (interactive
   (let* ((method (ringing:read-method-name "Test a lead of: "))
	  (manual-bell (ringing:read-number-up-to
			"Place bell"
			(change-ringing-method-nbells
			 (ringing:find-named-method method)))))
     (list method manual-bell)))
  (message "Try ringing place %d of %s" place-bell (ringing:method-name method))
  (let ((user-mistakes:teaching-info (user-mistakes:get-teaching-data (ringing:find-named-method method) place-bell))
	(ringing:show-lines-with-lines ringing:lines-between-rows-when-teaching))
    (setf (user-mistakes:lead-which-row user-mistakes:teaching-info) 0)
    (incf (user-mistakes:lead-attempts user-mistakes:teaching-info))
    (let ((result
	   (teach-ringing:do-one-lead method place-bell nil nil)))


      (setq ringing:prev-place 2)		; for now!



      result)))

(defun ringing:try-user-on-leads (method start-place-bell end-place-bell)
  "Let the user try METHOD on place bells from START-PLACE-BELL to END-PLACE-BELL."
  (interactive
   (let* ((method (ringing:read-method-name "Test some leads of: "))
	  (manual-bell-first (ringing:read-number-up-to
			"Place bell to start testing"
			(change-ringing-method-nbells
			 (ringing:find-named-method method))))
	  (manual-bell-last (ringing:read-number-up-to
			"Place bell to finish testing"
			(change-ringing-method-nbells
			 (ringing:find-named-method method)))))
     (list method manual-bell-first manual-bell-last)))
  (let ((place-bell start-place-bell))
    (while (<= place-bell end-place-bell)
      (ringing:try-user-on-lead method place-bell)
      (incf place-bell))))

(defun read-event-in (acceptable-events)
  "Read an event that is one of ACCEPTABLE-EVENTS"
  (let ((event (read-event)))
    (while (not (memq event acceptable-events))
      (message "Must enter one of: %s"
	       (mapconcat (function
			   (lambda (char)
			     (format (if (integerp char)
					 "\"%c\""
				       "\"%s\"")
				     char))) acceptable-events ", "))
      (setq event (read-event)))
    event))

(defun ringing:teach-lead-of-method (method-name manual-bell)
  "Show the user the lead of METHOD-NAME starting with MANUAL-BELL, then let them try it themselves."
  (interactive
   (let* ((method (ringing:read-method-name "Ring a lead of: "))
	  (manual-bell (ringing:read-number-up-to
			"Place bell"
			(change-ringing-method-nbells
			 (ringing:find-named-method method)))))
     (list method manual-bell)))
  (save-window-excursion
    (let* ((method (ringing:find-named-method method-name))
	   (ringing:show-lines-with-lines nil))
      (when teach-ringing:preview-leads
	(teach-ringing:do-one-lead method manual-bell nil nil))
      (if (and teach-ringing:preview-leads
	       (eq (progn
		     (message "Press space to try ringing this yourself")
		     (read-event-in '(up ? )))
		   'up))
	  'user-stand
	(ringing:try-user-on-lead method-name manual-bell)))))

(defun ringing:teach-random-leads-of-method (method-name)
  "Show the user random leads of METHOD-NAME, then let them try them themselves."
  (interactive
   (list (ringing:read-method-name "Ring random leads of: ")))
  (let ((method (ringing:find-named-method method-name))
	(result nil))
    (while (not (eq result 'user-stand))
      (setq result (ringing:teach-random-lead-of-method method)))
    (message "End of %s practice" (ringing:method-name method))))

(defvar teach-ringing:faults-keymap (make-keymap "Faults list"))
(suppress-keymap teach-ringing:faults-keymap)
(define-key teach-ringing:faults-keymap " " 'teach-ringing:faults-done)
(define-key teach-ringing:faults-keymap "q" 'teach-ringing:faults-quit)

(defun teach-ringing:faults-done ()
  "Show that you've finished looking at the faults buffer."
  (interactive)
  (throw 'faults-done t))

(defun teach-ringing:faults-quit ()
  "Show that you've finished looking at the faults buffer, and want to stop the learning session."
  (interactive)
  (throw 'faults-done nil))

(defun teach-ringing:make-fault-display-buffer ()
 "Make a buffer for displaying the user's mistakes."
    (switch-to-buffer (ringing:get-buffer-create "*Faults so far for this lead*"))
    (erase-buffer)
    (teach-ringing:fault-display-mode))

(defun teach-ringing:fault-display-mode ()
  "Major mode for displaying user mistakes in learning a change-ringing method.
You can move up and down the buffer, and, in the normal context of the buffer's
use, press SPC to finish looking at your mistakes or q to finish the learning
session."
  (kill-all-local-variables)
  (use-local-map teach-ringing:faults-keymap)
  (setq major-mode 'ringing-faults
	mode-name "Ringing Faults"))

(defun teach-ringing:display-lead-faults (method place-bell)
  "Display the faults for METHOD PLACE-BELL."
  (save-window-excursion
    (let ((method-teaching-data (user-mistakes:get-teaching-data method))
	  (teaching-data (user-mistakes:get-teaching-data method place-bell)))
      (teach-ringing:make-fault-display-buffer)
      (teach-ringing:display-one-lead-teaching-data method-teaching-data place-bell)
      (goto-char (point-min))
      (delete-other-windows)
      (message "%d faults that time; press space to continue"
	       (user-mistakes:lead-latest-faults teaching-data))
      (if nil
	  (if (eq (read-event-in '(up ? ))
		  'up)
	      (throw 'quick-quit 'user-stand))
	(progn
	  (catch 'faults-done
	    (recursive-edit)
	    t))))))

(defvar teach-ringing:tolerable-faults 0
  "*How many faults per lead to allow while counting the lead as correctly rung.")

(defvar teach-ringing:required-perfects-seen 2
  "*How many times the user must ring a lead correctly before not being shown it
before each try, in ringing:teach-each-lead-of-method-until-perfect.")

(defvar teach-ringing:required-perfects-unseen 2
  "*How many times the user must ring a lead correctly without being shown it
before each try, before they can go on to the next lead in ringing:teach-each-lead-of-method-until-perfect.")

(defun ringing:teach-lead-until-perfect (method place-bell &optional more-leads-to-come)
  "Get the user to try METHOD, PLACE-BELL until they can get it right."
  (interactive
   (let* ((method (ringing:read-method-name "Teach a lead of: "))
	  (manual-bell (ringing:read-number-up-to
			"Place bell"
			(change-ringing-method-nbells
			 (ringing:find-named-method method)))))
     (list method manual-bell)))
  (let* ((faults 0)
	 (teach-ringing:preview-leads t)
	 (perfects 0)
	 (result nil)
	 (method-data (ringing:find-named-method method)))

    (while (and (< perfects (if teach-ringing:preview-leads
				teach-ringing:required-perfects-seen
			      teach-ringing:required-perfects-unseen))
		(not (eq result 'user-stand)))

      (let ((first-attempt t))
	(while (or first-attempt
		   (and (> faults teach-ringing:tolerable-faults)
			(not (eq result 'user-stand))))

	  (setq result (ringing:teach-lead-of-method method-data place-bell)
		faults (user-mistakes:lead-latest-faults (user-mistakes:get-teaching-data method place-bell))
		first-attempt nil)

	  (when (> faults teach-ringing:tolerable-faults)
            (teach-ringing:display-lead-faults method place-bell))))

      (incf perfects)

      (let ((ok-string (if (zerop teach-ringing:tolerable-faults) "correct" "satisfactory")))
	(if teach-ringing:preview-leads
	    (if (< perfects teach-ringing:required-perfects-seen)
		(ringing:message "%d %s attempts while being taught -- %d more required"
				 perfects ok-string
				 (- teach-ringing:required-perfects-seen perfects))
	      (if (> teach-ringing:required-perfects-unseen 0)
		  (ringing:message "%d %s attempts while being taught; going on to test you (no preview of leads)"
				   perfects ok-string)
		(ringing:message "%d %s attempts while being taught"
				 perfects ok-string)))
	  (if (< perfects teach-ringing:required-perfects-unseen)
	      (ringing:message "%d %s attempts while being tested -- %d more required"
			       perfects ok-string
			       (- teach-ringing:required-perfects-unseen
				  perfects))
	    (ringing:message "%d %s attempts while being tested%s"
			     perfects ok-string
			     (if more-leads-to-come
				 (format "; going on to next lead%s"
					 (if (and teach-ringing:test-previous-leads
						  (> place-bell 2))
					     " ... but first... "
					   ""))
			       "")))))

      (when (and (>= perfects teach-ringing:required-perfects-seen)
		 teach-ringing:preview-leads)
	(setq perfects 0
	      teach-ringing:preview-leads nil)))
    result))

(defvar teach-ringing:test-previous-leads t
  "*Whether ringing:teach-each-lead-of-method-until-perfect should go back
through all the previous leads each time it moves on to the next lead,
to make sure you haven't forgotten them.
If numeric, go back through that number of leads rather than all of them.")

(defun ringing:teach-lead-and-test-leads-so-far (method place-bell max-bell)
  "For METHOD teach the user PLACE-BELL (up to MAX-BELL) then check they haven't forgotten previous place bells."
  (prog1
      (ringing:teach-lead-until-perfect method
					place-bell
					(< place-bell max-bell))

    (when (and teach-ringing:test-previous-leads
	       (> place-bell 2))
      (let ((back-from (if (numberp teach-ringing:test-previous-leads)
			   (max (- place-bell teach-ringing:test-previous-leads) 2)
			 2)))
	(ringing:message "Going back through leads %d onward" back-from)
	(ringing:try-user-on-leads method
				   back-from
				   place-bell)))))

(defun ringing:teach-each-lead-of-method-until-perfect (method-name &optional starting-from)
  "Teach user each lead in turn of METHOD-NAME, repeating each until reliable.
With optional prefix numeric arg, start from that place bell, otherwise from 2."
  (interactive
   (let* ((initial-place-bell (if current-prefix-arg
				  (prefix-numeric-value current-prefix-arg)
				2))
	  (method-name (ringing:read-method-name "Teach each lead of: ")))
     (list method-name initial-place-bell)))

  (save-window-excursion
    (delete-other-windows )
    (let* ((teach-ringing:in-teach-until-perfect t)
	   (method (ringing:find-named-method method-name))
	   (ringing:auto-ring-inter-row-delay 0)
	   (n (change-ringing-method-nbells method))
	   (place-bell starting-from)
	   (result nil))

      (user-mistakes:with-previous-mistakes-hidden
       (catch 'quick-quit
	 (require 'mSiRiL-mode)
	 (let ((mSiRiL-mode:ringing-in-progress-format "Previewing all of %s")
	       (ringing:show-lines-with-lines nil)
	       (ringing:auto-ring-inter-row-delay nil)
	       )
	   (mSiRiL-mode:lines-for-method method 2 'switch-to-buffer))
	 (message "Press space to continue")
	 (if (eq (read-event-in '(up ? ))
		 'up)
	     (throw 'quick-quit 'user-stand))

	 (while (and (<= place-bell n) (not (eq result 'user-stand)))
	   (setq result
		 (ringing:teach-lead-and-test-leads-so-far method place-bell n))
	   (incf place-bell))

	 (message "Now try a plain course -- press space to start")
	 (if (eq (read-event-in '(up ? ))
		 'up)
	     (throw 'quick-quit 'user-stand))
	 (ringing:ring-plain-course method 2)))))

  (message "End of %s practice" (ringing:method-name method-name)))

(defun ringing:teach-random-methods-from-file (file)
  "Show the user random leads from random methods in FILE, and get them to try them."
  (interactive "fTry random leads from file: ")
  (let* ((file-data (microSiRiL:parse-file-superficially file))
	 (n-methods (length file-data))
	 (result nil))
    (while (not (eq result 'user-stand))
      (let ((method (car (nth (random n-methods) file-data))))
	(setq result (ringing:teach-random-lead-of-method method)))))
  (message "End of %s practice" file))

(defun ringing:teach-random-methods-from-group (group)
  "Show the user random leads from random methods in GROUP, and get them to try them."
  (interactive
   (let ((completion-ignore-case t))
     (list
      (completing-read "Try random leads from group: "
		       (method-groups:all-groups)
		       nil t))))
  (let* ((group-methods (method-groups:group-methods group))
	 (n-methods (length group-methods))
	 (result nil))
    (while (not (eq result 'user-stand))
      (let ((method (car (nth (random n-methods) group-methods))))
	(setq result (ringing:teach-random-lead-of-method method)))))
  (message "End of %s practice" (method-groups:group-name group)))

(defvar ringing:quiz-acceptable-faults 3
  "The number of faults to allow while quizzing the user on a group.
If this number is exceeded in a lead, the user is re-taught that lead.")

(defvar ringing:quiz-between-leads nil
  "What to do between the leads of ringing quizzes.
If this is a function, it is called, and the quiz finishes if the function
returns nil; if it is a buffer, that buffer is displayed briefly (so, for
example, you could be doing a quiz while waiting for a compilation to finish.
If ringing:quiz-at-stage is called from a shell-mode buffer, this is set
automatically to that buffer.")

(defvar ringing:quiz-between-leads-delay-time 1
  "How long to show the user a buffer between quiz leads.")

(defun ringing:quiz-random-methods-from-group (group &optional n)
  "Test the user on random leads from GROUP.
With optional argument N neither nil nor 1, stop after that many leads."
  (interactive
   (let ((completion-ignore-case t)
	 (limit (prefix-numeric-value current-prefix-arg)))
     (list
      (completing-read "Try random leads from group: "
		       (method-groups:all-groups)
		       nil t)
      limit)))
  (random t)
  (let* ((group-methods (method-groups:group-methods group))
	 (n-methods (length group-methods))
	 (countdown (if (and (not (null n))
			     (not (= n 1)))
			n
		      -1))
	 (result nil))
    (while (and (not (eq result 'user-stand))
		(not (zerop countdown)))
      (let* ((method (car (nth (random n-methods) group-methods)))
	     (place-bell (ringing:random-working-bell method)))
	(setq result (ringing:try-user-on-lead
		      method place-bell))
	(ringing:message "%d clashes" ringing:clashes)
	(when (and t (>= ringing:clashes ringing:quiz-acceptable-faults))
	  (ringing:message "Too many faults -- re-teaching this")
	  (ringing:teach-lead-until-perfect method place-bell)))
      (decf countdown)
      (cond
       ((or (and ringing:quiz-between-leads (symbolp ringing:quiz-between-leads))
	    (consp ringing:quiz-between-leads))
	(when (null (funcall ringing:quiz-between-leads))
	  (setq result 'user-stand)))
       ((bufferp ringing:quiz-between-leads)
	(save-window-excursion
	  (switch-to-buffer ringing:quiz-between-leads)
	  (sit-for ringing:quiz-between-leads-delay-time))))))
  (message "End of %s quiz" (method-groups:group-name group)))

(defun teach-ringing:group-teaching-data (group)
  "Make sure that all the teaching record information is present for GROUP at STAGE.
and return it as an array of user-mistakes:lead structures."
    (let* ((group-methods (method-groups:group-methods group))
	   (teaching-data nil))
      (dolist (method group-methods)
	(let ((i 2))
	  (while (<= i (change-ringing-method-nbells (car method)))
	    (push (user-mistakes:get-teaching-data (car method) i)
		  teaching-data)
	    (incf i))))
      (coerce teaching-data 'array)))

(defun ringing:choose-call-randomly (current-method group)
  "Return a random call."
  (let* ((include-singles
	  nil
	  )
	 (next-method current-method)
	 (choice-max (if include-singles 4 3))
	 (action (random choice-max)))
    (message "Action %d from up to %d" action choice-max)

    (case action
      (0 nil)
      (1 (setq ringing:next-lead-end-action (list 'bob current-method))
	 (message "Bob at end of this lead")
	 )
      (2 (setq next-method (method-group:random-method-from-group
			    group
			    current-method)
	       ringing:next-lead-end-action (list 'change-of-method (ringing:method-name next-method)))
	 (message "Next method %s after this lead" (ringing:method-name next-method))
	 )
      (3 (setq ringing:next-lead-end-action (list 'single current-method))
	 (message "Single at end of this lead")
	 ))

    ringing:next-lead-end-action))

(defun ringing:mad-conductor-helper-function (row
					      remaining-changes
					      method-holder
					      rounds
					      place-marker
					      touch-spec
					      count-holder
					      truth-holder
					      group)
  "Helper function for constrainedly-random conducting."
  (incf ringing:plain-course-row-count)
  (incf (car count-holder))
  (declare '(special group))
  (if (string= row rounds)
      nil
    (if remaining-changes
	t
      (let ((method (car method-holder)))
	(cons
	 row
	 (case (car place-marker)
	   ((lead-end)

	    (when ringing:lead-end-callback
	      (apply ringing:lead-end-callback
		     row
		     remaining-changes
		     method
		     ringing:lead-end-callback-args))

	    (rplaca place-marker 'work)
	    (change-ringing-method-work method))
	   ((work)
	    (rplaca place-marker 'lead-end)
	    (let* ((this-call (ringing:choose-call-randomly method group)))
	      (case (car this-call)
		(change-of-method
		 (setq method (ringing:find-named-method (cadr this-call)))
		 ;; (setq user-mistakes:teaching-info (user-mistakes:get-teaching-data method start))
		 (rplaca method-holder method)


		 (setf (user-mistakes:lead-which-row user-mistakes:teaching-info) 0)
		 (incf (user-mistakes:lead-attempts user-mistakes:teaching-info))

		 (change-ringing-method-plain-lead-end method))
		(bob

		 (setf (user-mistakes:lead-which-row user-mistakes:teaching-info) 0)
		 (incf (user-mistakes:lead-attempts user-mistakes:teaching-info))


		 (change-ringing-method-bob method))

		(setf (user-mistakes:lead-which-row user-mistakes:teaching-info) 0)
		(incf (user-mistakes:lead-attempts user-mistakes:teaching-info))

		(single


		 (setf (user-mistakes:lead-which-row user-mistakes:teaching-info) 0)
		 (incf (user-mistakes:lead-attempts user-mistakes:teaching-info))

		 (change-ringing-method-single method))
		(nil 

		 (setf (user-mistakes:lead-which-row user-mistakes:teaching-info) 0)
		 (incf (user-mistakes:lead-attempts user-mistakes:teaching-info))

		 nil)
		(plain


		 (setf (user-mistakes:lead-which-row user-mistakes:teaching-info) 0)
		 (incf (user-mistakes:lead-attempts user-mistakes:teaching-info))
		 (change-ringing-method-plain-lead-end method))
		(t

		 (setf (user-mistakes:lead-which-row user-mistakes:teaching-info) 0)
		 (incf (user-mistakes:lead-attempts user-mistakes:teaching-info))
		 (change-ringing-method-plain-lead-end method)))))))))))

(defun ringing:teach-with-mad-conductor (group)
  "Ring randonly but connectedly from GROUP.
At the end of each lead, a plain lead end, bob, single, or change of method
is called."
  (interactive
   (let ((completion-ignore-case t))
     (list
      (completing-read "Ring random touch from group: "
		       (method-groups:all-groups)
		       nil t))))
  (when (yes-or-no-p "Note: this routine is under development: try it anyway? ")
    (require 'touches)
    (let* ((ringing:touch-buffer-header-format (format "Random touch of %s" group))
	   (ringing:touch-buffer-name-format (format "*random touch of %s*" group))
	   (method (ringing:find-named-method (method-group:random-method-from-group group)))
	   (teach-ringing:preview-leads nil)
	   (start (ringing:random-working-bell method))
	   (result nil)


					;; fiddling with the calls!
	   (ringing:call-at-remaining-rows 2)



	   (user-mistakes:teaching-info (user-mistakes:get-teaching-data method start)))
      (setf (user-mistakes:lead-which-row user-mistakes:teaching-info) 0)
      (incf (user-mistakes:lead-attempts user-mistakes:teaching-info))
      (message "Go %s!" (ringing:method-name method))
      (ringing:ring-touch (ringing:method-name method)
			  start
			  nil
			  'ringing:mad-conductor-helper-function
			  'teach-ringing:display-touch-function
			  (list group)
			  nil)
      (message "End of %s practice" group))))

(defun times-string (n)
  "Return a cardinal string for N."
  (cond
   ((eq n 1) "once")
   ((eq n 2) "twice")
   ( t(format "%d times" n))))

(defun teach-ringing:display-one-lead-teaching-data (method-teaching-data place-bell &optional own-buffer)
  "Display the teaching data for METHOD-TEACHING-DATA and PLACE-BELL."
  (interactive
   (let* ((method (completing-read "Display teaching data for a lead of: "
				   user-mistakes:methods
				   nil t))
	  (manual-bell (ringing:read-number-up-to
			"Place bell"
			(change-ringing-method-nbells
			 (ringing:find-named-method method)))))
     (list (user-mistakes:get-teaching-data method) manual-bell t)))
  (let* ((method (user-mistakes:method-method method-teaching-data))
	 (name (ringing:method-name method))
	 (leads (user-mistakes:method-leads method-teaching-data))
	 (lead-data (aref leads place-bell)))
    (when own-buffer
      (switch-to-buffer-other-window
       (ringing:get-buffer-create
	(format "*Teaching data for place %d of %s*" place-bell name)))
      (erase-buffer))
    (message "Preparing report: method %s place %d..." name place-bell)
    (when (and lead-data (not (zerop (user-mistakes:lead-attempts lead-data))))
      (let ((fault-array (user-mistakes:lead-row-faults lead-data))
	    (attempts (user-mistakes:lead-attempts lead-data)))
	(insert (format " Place %d of %s: %d attempts (%d perfect)\n"
			place-bell name attempts
			(user-mistakes:lead-perfects lead-data)))
	(insert (format "  faults last time: %d; total faults so far: %d;\n"
			(user-mistakes:lead-latest-faults lead-data)
			(user-mistakes:lead-total-faults lead-data)
			))
	(let ((imperfects (- attempts (user-mistakes:lead-perfects lead-data))))
	  (unless (zerop imperfects)
	    (insert (format "  average faults per imperfect lead: %f\n"
			    (/ (float (user-mistakes:lead-total-faults lead-data))
			       (float imperfects))))))
	(let ((user-mistakes:teaching-info (user-mistakes:get-teaching-data method place-bell))
	      (ringing:show-lines-with-lines t))
	  (setf (user-mistakes:lead-which-row user-mistakes:teaching-info) 0)
	  (teach-ringing:do-one-lead method place-bell nil t)
	  (insert "\n"))))))

(defun teach-ringing:display-all-teaching-data ()
  "Display all the teaching data."
  (interactive)
  (switch-to-buffer (ringing:get-buffer-create "*Teaching data*"))
  (erase-buffer)
  (let ((method-pairs user-mistakes:methods))
    (while method-pairs
      (let* ((name (car (car method-pairs)))
	     (method-teaching-data (cdr (car method-pairs))))
	(insert name "\n")
	(let ((i 1)
	      (n (change-ringing-method-nbells
		  (user-mistakes:method-method method-teaching-data))))
	  (while (<= i n)
	    (teach-ringing:display-one-lead-teaching-data method-teaching-data i)
	    (incf i))))
      (setq method-pairs (cdr method-pairs))))
  (message "")
  (goto-char (point-min)))

(defun teach-ringing:forget-mistakes ()
  "Forget the pupil's record so far."
  (interactive)
  (setq user-mistakes:methods nil))

;;; end of teacher.el
